//-------------------------------------------------------------------------------------------------
// ParseBuffer.hpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_ParseBuffer_
#define _include_ParseBuffer_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.hpp"
#include "Vector4.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		ParseBuffer
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class ParseBuffer
{
private:
public:
	const char*		_bufferName;
	const char*		_buffer;
	int				_bufferSize;
	const char*		_read;
	int				_lineNum;
	const char*		_lineStart;

	void _SetBufferSize();
	void _CopyUpToScan( const char*& read, char*& write, const char* scan );
	void _SyntaxError( const char* errorDesc );

public:
	ParseBuffer( const char* buffer, int bufferSize = -1, const char* bufferName = "unnamed" );
	ParseBuffer( const std::string& buffer, int bufferSize = -1, const char* bufferName = "unnamed" );
	~ParseBuffer();

	void SkipWhitespace();
	void StripComments();
	int ParseInt();
	float ParseFloat();
	double ParseDouble();
	void ParseQuotedString( char* destBuffer, int destBufferSize, bool allowLineWrap = false );
	void ParseQuotedString( std::string& destBuffer, bool allowLineWrap = false );
	void ParseContiguousString( char* destBuffer, int destBufferSize );
	void ParseStringToEndOfLine( char* destBuffer, int destBufferSize );
	Vector2 ParseVector2();
	Vector3 ParseVector3();
	Vector4 ParseVector4();
	void ParseExpectedToken( char token );
	void ParseExpectedToken( const char* token );
	char AdvanceToNextToken();

	friend class ParseFile;
};


//-------------------------------------------------------------------------------------------------
// SkipWhitespace
//-------------------------------------------------------------------------------------------------
inline void ParseBuffer::SkipWhitespace()
{
	while( isspace( *_read ) )
	{
		if( *_read == '\n' )
		{
			_lineNum ++;
			_lineStart = _read + 1;
		}

		_read ++;
	}
}





#endif // _include_ParseBuffer_
