//-------------------------------------------------------------------------------------------------
// ParseFile.hpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_ParseFile_
#define _include_ParseFile_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.hpp"
#include "ParseBuffer.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		ParseFile
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class ParseFile
{
private:
public:
	ParseBuffer*	_parseBuffer;		// the actual parsing mechanism once the file has been read in
	const char*		_filePath;			// name of file being parsed
	FILE*			_fileHandle;		// FILE* of file being parsed
//	bool			_ownsFileHandle;	// if true, ParseFile opened _fileHandle and should close it

	void _OpenFileByName( const char* filePath );
	void _LoadFileIntoBufferAndClose( bool closeFileHandle );

public:
	ParseFile( const char* filePath );
	ParseFile( const std::string& filePath );
	ParseFile( FILE* fileHandle );
	~ParseFile();

	void SkipWhitespace();
	void StripComments();
	int ParseInt();
	float ParseFloat();
	double ParseDouble();
	void ParseQuotedString( char* destBuffer, int destBufferSize, bool allowLineWrap = false );
	void ParseQuotedString( std::string& destBuffer, bool allowLineWrap = false );
	void ParseContiguousString( char* destBuffer, int destBufferSize );
	void ParseStringToEndOfLine( char* destBuffer, int destBufferSize );
	Vector2 ParseVector2();
	Vector3 ParseVector3();
	Vector4 ParseVector4();
	void ParseExpectedToken( char token );
	void ParseExpectedToken( const char* token );
	char AdvanceToNextToken();
};




#endif // _include_ParseFile_
