//-------------------------------------------------------------------------------------------------
// Texture.hpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// A Texture object is created from an Image and may be used in-game by OpenGL or TheRenderer.
//-------------------------------------------------------------------------------------------------
#ifndef _include_Texture_
#define _include_Texture_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.hpp"
#include "Image.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		Texture
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class Texture
{
private:
	static const dword_t TEXNUM_UNREGISTERED;
	static bool _isOpenGLReady;

	word_t			_width;						// number of pixels wide
	word_t			_height;					// number of pixels high
	byte_t			_bytesPerPixel;				// bytes per pixel; usually 3 or 4
	GLenum			_internalFormat;			// format specification for how opengl should store the image internally
	GLuint			_glTexNum;					// opengl texture name (number); TEXNUM_UNREGISTERED if unregistered

	void _InitFromImage( const Image& image );
	void _CreateOpenGLTexture( const Image& image );

	static void _SetIsOpenGLReady( bool isOpenGLReady ) { _isOpenGLReady = isOpenGLReady; }

public:

	Texture( const std::string& filePath );
	Texture( const Image& image );
	~Texture();

	inline int GetWidth() const { return _width; }
	inline int GetHeight() const { return _height; }
	inline GLuint GetGLTexNum() const { return _glTexNum; }

	friend class TheRenderer;
};




#endif // _include_Texture_
