//-------------------------------------------------------------------------------------------------
// TheGameInterfaceAbstract.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "TheGameInterfaceAbstract.hpp"
#include "engineAssert.hpp"
using namespace std;


//-------------------------------------------------------------------------------------------------
// Subsystem name macro (for assertions, etc)
//-------------------------------------------------------------------------------------------------
#define SUBSYSTEM_NAME "TheGameInterfaceAbstract"


//-------------------------------------------------------------------------------------------------
// Static member initialization
//-------------------------------------------------------------------------------------------------
STATIC TheGameInterfaceAbstract*	TheGameInterfaceAbstract::_theGameInterfaceAbstract = NULL;
STATIC bool							TheGameInterfaceAbstract::_isConstructed = false;



//-------------------------------------------------------------------------------------------------
// Destructor()
//-------------------------------------------------------------------------------------------------
TheGameInterfaceAbstract::~TheGameInterfaceAbstract()
{
	/// Make sure the singleton object knew it was constructed before destroying
	VALIDATE_MSG0( _isConstructed, SUBSYSTEM_NAME" was destroyed unexpectedly (internal engine error!)" );

	/// Set un-constructed
	_isConstructed = false;
}


//-------------------------------------------------------------------------------------------------
// Default Constructor
//-------------------------------------------------------------------------------------------------
TheGameInterfaceAbstract::TheGameInterfaceAbstract()
{
	/// Make sure the singleton object doesn't get constructed twice
	VALIDATE_MSG0( !_isConstructed, SUBSYSTEM_NAME" was already constructed once - cannot have multiple instances!" );

	/// Store a pointer to the derived game interface object being constructed
	_theGameInterfaceAbstract = this;

	/// Set construction complete
	_isConstructed = true;
}


//-------------------------------------------------------------------------------------------------
// ProcessCommandLine
// 
// Implement this function in the derived game class to handle (or change) command line options.
// 
// <cmdLine> is a string containing the application's command line as it was specified at
//	run time.  The game may feel free to parse and/or modify the contents of the string at
//	its discretion; the modified string will then be treated as the effective command line
//	by the application internally.
//-------------------------------------------------------------------------------------------------
VIRTUAL void TheGameInterfaceAbstract::ProcessCommandLine( string cmdLine )
{
	ASSERT_MSG1( _isConstructed, SUBSYSTEM_NAME"::ProcessCommandLine( %s ) failed; "SUBSYSTEM_NAME" was not constructed.", cmdLine.c_str() );
	ASSERT_MSG1( _theGameInterfaceAbstract, SUBSYSTEM_NAME"::ProcessCommandLine( %s ) failed; "SUBSYSTEM_NAME" was constructed but not registered (internal engine error).", cmdLine.c_str() );

	UNUSED_ARG cmdLine;
}


//-------------------------------------------------------------------------------------------------
// Singleton
//-------------------------------------------------------------------------------------------------
STATIC TheGameInterfaceAbstract* TheGameInterfaceAbstract::Singleton()
{
	ASSERT_MSG0( _isConstructed, SUBSYSTEM_NAME"::Singleton( %s ) failed; "SUBSYSTEM_NAME" has not been constructed." );
	ASSERT_MSG0( _theGameInterfaceAbstract, SUBSYSTEM_NAME"::Singleton( %s ) failed; "SUBSYSTEM_NAME" was constructed but not registered (internal engine error)." );

	return _theGameInterfaceAbstract;
}


//-------------------------------------------------------------------------------------------------
// PreInit
//-------------------------------------------------------------------------------------------------
void TheGameInterfaceAbstract::PreInit()
{
}


//-------------------------------------------------------------------------------------------------
// PostInit
//-------------------------------------------------------------------------------------------------
void TheGameInterfaceAbstract::PostInit()
{
}


//-------------------------------------------------------------------------------------------------
// Shutdown
//-------------------------------------------------------------------------------------------------
void TheGameInterfaceAbstract::Shutdown()
{
}


//-------------------------------------------------------------------------------------------------
// RunFrame
//-------------------------------------------------------------------------------------------------
void TheGameInterfaceAbstract::RunFrame()
{
}


//-------------------------------------------------------------------------------------------------
// ProcessInputEvent
//-------------------------------------------------------------------------------------------------
VIRTUAL const bool TheGameInterfaceAbstract::ProcessInputEvent( const TheInputSystem::inputEventType_e eventType, const byte_t key )
{
	ASSERT_MSG0( _isConstructed, SUBSYSTEM_NAME"::ProcessInputEvent( %s ) failed; "SUBSYSTEM_NAME" has not been constructed." );
	ASSERT_MSG0( _theGameInterfaceAbstract, SUBSYSTEM_NAME"::ProcessInputEvent( %s ) failed; "SUBSYSTEM_NAME" was constructed but not registered (internal engine error)." );

	UNUSED_ARG eventType;
	UNUSED_ARG key;

	return false;
}


