//-------------------------------------------------------------------------------------------------
// ThePhysicsSystem.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "ThePhysicsSystem.hpp"
#include "TheConsole.hpp"
#include "engineAssert.hpp"


//-------------------------------------------------------------------------------------------------
// Subsystem name macro (for assertions, etc)
//-------------------------------------------------------------------------------------------------
#define SUBSYSTEM_NAME "ThePhysicsSystem"


//-------------------------------------------------------------------------------------------------
// Static member initialization
//-------------------------------------------------------------------------------------------------
STATIC bool				ThePhysicsSystem::_isInitialized		= false;


//-------------------------------------------------------------------------------------------------
// constructor
//-------------------------------------------------------------------------------------------------
ThePhysicsSystem::ThePhysicsSystem()
{
}


//-------------------------------------------------------------------------------------------------
// destructor
//-------------------------------------------------------------------------------------------------
ThePhysicsSystem::~ThePhysicsSystem()
{
	/// Make sure the subsystem was shut down before being destroyed
	VALIDATE_MSG0( !_isInitialized, SUBSYSTEM_NAME" was destroyed while running; Shutdown() should be called first!" );
}


//-------------------------------------------------------------------------------------------------
// Startup
//-------------------------------------------------------------------------------------------------
void ThePhysicsSystem::Startup()
{
	theConsole.Printf( "::: Initialization : "SUBSYSTEM_NAME" (starting)\n" );

	/// Make sure the subsystem doesn't get initialized twice
	VALIDATE_MSG0( !_isInitialized, SUBSYSTEM_NAME" was initialized twice; either a second "SUBSYSTEM_NAME" was started or Startup() was called on an already-running system." );

	/// Set initialization complete
	_isInitialized = true;
	theConsole.Printf( "::: Initialization : "SUBSYSTEM_NAME" (finished)\n" );
}


//-------------------------------------------------------------------------------------------------
// Shutdown
//-------------------------------------------------------------------------------------------------
void ThePhysicsSystem::Shutdown()
{
	theConsole.Printf( "::: Shutdown : "SUBSYSTEM_NAME" (starting)\n" );

	/// De-initialize the system
	_isInitialized = false;
	theConsole.Printf( "::: Shutdown : "SUBSYSTEM_NAME" (finished)\n" );
}


