//-------------------------------------------------------------------------------------------------
// engineCommon.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineCommon.hpp"
#include "engineAssert.hpp"



//-------------------------------------------------------------------------------------------------
// ::Sprintf (like the standard C library one, only length-overrun-safe)
//-------------------------------------------------------------------------------------------------
int Sprintf( char* buffer, size_t maxLength, const char* format, ... )
{
	va_list		args;
	int			return_value;

	ASSERT_MSG0( format, "Sprintf() : the <format> string was NULL!" );
	ASSERT_MSG1( buffer, "Sprintf( \"%s\", ... ) : <buffer> was NULL!", format );
	ASSERT_MSG2( maxLength > 0, "Bad length %i specified for Sprintf; format string was \"%s\"", maxLength, format );

	/// Write formatted string into <buffer>
	va_start( args, format );
	return_value = _vsnprintf( buffer, maxLength, format, args );
	va_end( args );
	buffer[ maxLength - 1 ] = '\0';

	/// Follow the same return-value convention as _vsnprintf()
	return( return_value );
}


//-------------------------------------------------------------------------------------------------
// ::DebugPrintf (like OutputDebugString, only with sprintf-style formatting)
//-------------------------------------------------------------------------------------------------
void DebugPrintf( const char* format, ... )
{
	const unsigned int	TEXT_BUFFER_SIZE = 2048;
	char				text[ TEXT_BUFFER_SIZE ];
	va_list				args;

	ASSERT_MSG0( format, "DebugPrintf() : the <format> string was NULL!" );

	/// Write formatted string into <buffer>
	va_start( args, format );
	_vsnprintf( text, TEXT_BUFFER_SIZE, format, args );
	va_end( args );
	text[ TEXT_BUFFER_SIZE - 1 ] = '\0';

	OutputDebugString( text );
}


//-------------------------------------------------------------------------------------------------
// LoadBinaryFileToBuffer
//-------------------------------------------------------------------------------------------------
unsigned char* LoadBinaryFileToBuffer( const std::string& filePath, int& byteSize )
{
	/// Open the file
	FILE* file = fopen( filePath.c_str(), "rb" );
	VALIDATE_MSG1( file, "LoadBinaryFileToBuffer FAILED; could not open file %s for reading", filePath.c_str() );

	/// Determine file size
	fseek( file, 0, SEEK_END );
	int fileSize = ftell( file );
	rewind( file );

	/// Allocate memory for buffer
	unsigned char* byteBuffer = new unsigned char[ fileSize + 1 ];
	VALIDATE( byteBuffer );

	/// Read file contents into buffer
	fread( byteBuffer, 1, fileSize, file );	

	/// Close the file
	fclose( file );

	byteSize = fileSize;
	return byteBuffer;
}



