//-------------------------------------------------------------------------------------------------
// Grid.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "Grid.hpp"


//-------------------------------------------------------------------------------------------------
// Destructor()
//-------------------------------------------------------------------------------------------------
Grid::~Grid()
{
}


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
Grid::Grid( float gridSize, float minGridSize, float maxGridSize, bool snapToGrid )
:
m_gridSize( gridSize ),
m_minGridSize( minGridSize ),
m_maxGridSize( maxGridSize ),
m_snapToGrid( snapToGrid )
{
}



//-------------------------------------------------------------------------------------------------
// FindClosestGridPointToWorldCoordinates
//-------------------------------------------------------------------------------------------------
const Vector2 Grid::FindClosestGridPointToWorldCoordinates( const Vector2& worldCoords )
{
	if( !m_snapToGrid )
	{
		return worldCoords;
	}

	Vector2 closestPoint( worldCoords );
	float halfGridSize = 0.5f * m_gridSize;
	closestPoint += Vector2( halfGridSize, halfGridSize );
	closestPoint /= m_gridSize;
	closestPoint.x = (float) floor( closestPoint.x );
	closestPoint.y = (float) floor( closestPoint.y );
	closestPoint *= m_gridSize;

	return closestPoint;
}


//-------------------------------------------------------------------------------------------------
// DecreaseGridSize
//-------------------------------------------------------------------------------------------------
void Grid::DecreaseGridSize()
{
	m_gridSize *= 0.5f;
	if( m_gridSize < m_minGridSize )
	{
		m_gridSize = m_minGridSize;
	}

	m_gridSize /= m_minGridSize;
	m_gridSize = (float) floor( m_gridSize );
	m_gridSize *= m_minGridSize;
}


//-------------------------------------------------------------------------------------------------
// IncreaseGridSize
//-------------------------------------------------------------------------------------------------
void Grid::IncreaseGridSize()
{
	m_gridSize *= 2.0f;
	if( m_gridSize > m_maxGridSize )
	{
		m_gridSize = m_maxGridSize;
	}

	m_gridSize /= m_minGridSize;
	m_gridSize = (float) floor( m_gridSize );
	m_gridSize *= m_minGridSize;
}


//-------------------------------------------------------------------------------------------------
// SetGridSize
//-------------------------------------------------------------------------------------------------
void Grid::SetGridSize( float newGridSize )
{
	m_gridSize = newGridSize;
	m_gridSize /= m_minGridSize;
	m_gridSize = (float) floor( m_gridSize );
	m_gridSize *= m_minGridSize;
}


//-------------------------------------------------------------------------------------------------
// GetGridEpsilon
//-------------------------------------------------------------------------------------------------
STATIC float Grid::GetGridEpsilon()
{
	return 0.5f * DEFAULT_MIN_GRID_SIZE; // FIXME: this min grid size is a mega-hack.  What to do?
}

