//-------------------------------------------------------------------------------------------------
// MapArea.hpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_MapArea_
#define _include_MapArea_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "MapLinedef.hpp"
#include "MapObject.hpp"
#include "gameCommon.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		MapArea
// 
///////////////////////////////////////////////////////////////////////////////////////////////////
class MapArea
{
private:
public:
	/// Members
	std::list<MapVertex*>	m_vertexList;
	std::list<MapLinedef*>	m_linedefList;
	std::list<MapObject*>	m_objectList;
	std::list<MapElement*>	m_highlightList;
	std::list<MapElement*>	m_selectionList;
	bool					m_isMapFlippedVertically;
	std::string				m_mapName;

	/// Constructors
	MapArea( const std::string& mapFilePath, bool isMapFlippedVertically );
	MapArea();
	~MapArea();

	/// Load / save
	bool SaveToFile( const std::string& mapFilePath );
	void SaveVertexesToFile( FILE* file );
	void SaveLinedefsToFile( FILE* file );
	void SaveObjectsToFile( FILE* file );
	void SavePropertiesToFile( const MapProperties& properties, FILE* file );
	bool PopulateFromFile( const std::string& mapFilePath );
	void ParseVertexes( ParseFile& mapFile );
	void ParseLinedefs( ParseFile& mapFile, int vertIndexOffset );
	void ParseObjects( ParseFile& mapFile );
	void ParseNewVertex( ParseFile& mapFile );
	void ParseNewLinedef( ParseFile& mapFile, int numInitialVerts );
	void ParseNewObject( ParseFile& mapFile );

	/// Highlight / select
	void ClearHighlight();
	void ClearSelection();
	void HighlightElement( MapElement* element );
	void DeleteHighlightedAndSelected();
	void ToggleSelectionOfHighlightedElements();
	void SelectElement( MapElement* element );
	void DeselectElement( MapElement* element );
	void TranslateSelection( const Vector2& translation );
	int NumSelectedElements();
	int NumSelectedElementsOfType( const MapElement::ElementType& type );
	void SetPropertyForHighlightedElements( const std::string& key, const std::string& value );
	void DeletePropertyForHighlightedElements( const std::string& key );

	/// Create / delete
	MapVertex* CreateNewVertex( const Vector2& pos, bool allowDuplicatePosition );
	MapLinedef* CreateNewLinedef( MapVertex* start, MapVertex* end );
	MapObject* CreateNewObject( const Vector2& pos );
	MapObject* CreateNewObject( const MapObject& objectToCopy, const Vector2& pos );
	void DeleteMapVertex( MapVertex* killMe, MapLinedef* killerLinedef = NULL );
	void DeleteMapLinedef( MapLinedef* killLinedef, MapVertex* killerVertex = NULL );
	void DeleteMapObject( MapObject* killObject );
	void DeleteMapElement( MapElement* element );
	void DeleteAllElements();

	/// Misc
	MapVertex* GetNthVertex( int vertIndex );
	int GetVertIndex( const MapVertex* vertex );
	MapVertex* IntegrateVertexIntoWorld( MapVertex* vertex );
	MapElement* FindClosestMapElementToWorldCoords( Vector2 worldCoords, MapElement::ElementType type, float& distSquaredToObject );
	MapElement* FindClosestMapElementToWorldCoords( Vector2 worldCoords, float& distSquaredToElement );
	MapLinedef* FindClosestMapLinedefToWorldCoords( Vector2 worldCoords, float& distSquaredToElement );
	MapVertex* FindClosestMapVertexToWorldCoords( Vector2 worldCoords, float& distSquaredToElement );
	MapObject* FindClosestMapObjectToWorldCoords( Vector2 worldCoords, float& distSquaredToElement );
};


//-------------------------------------------------------------------------------------------------
// Exported globals
//-------------------------------------------------------------------------------------------------
extern MapArea* g_currentArea;


#endif // _include_MapArea_
