//-------------------------------------------------------------------------------------------------
// Options.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "Options.hpp"
using namespace std;


//-------------------------------------------------------------------------------------------------
// Option defaults
//-------------------------------------------------------------------------------------------------
const float DEFAULT_MIN_VERTEX_SELECT_WORLD_DIST = 8.0f;
const float DEFAULT_MAX_XY_SELECT_SCREEN_DIST = 3.0f;
const Rgba DEFAULT_GRID_LINE_COLOR( 32, 32, 64, 255 );
const Rgba DEFAULT_GRID_DOT_COLOR( 64, 64, 128, 255 );
const Rgba DEFAULT_LINEDEF_COLOR( 128, 128, 128, 255 );
const Rgba DEFAULT_LINEDEF_HIGHLIGHT_COLOR_FG( 0, 0, 0, 255 );
const Rgba DEFAULT_LINEDEF_HIGHLIGHT_COLOR_BG( 255, 255, 0, 192 );
const Rgba DEFAULT_LINEDEF_SELECTION_COLOR_FG( 0, 0, 0, 255 );
const Rgba DEFAULT_LINEDEF_SELECTION_COLOR_BG( 255, 0, 0, 192 );
const Rgba DEFAULT_LINEDEF_HIGHLIGHT_SELECTION_COLOR_FG( 0, 0, 0, 255 );
const Rgba DEFAULT_LINEDEF_HIGHLIGHT_SELECTION_COLOR_BG( 255, 128, 0, 192 );
const Rgba DEFAULT_PROTO_LINEDEF_COLOR( 255, 0, 255, 255 );
const Rgba DEFAULT_XY_CLEAR_COLOR( 0, 0, 0, 255 );
const Rgba DEFAULT_CLOSEST_GRID_POINT_COLOR( 128, 128, 128, 255 );
const Rgba DEFAULT_VERTEX_COLOR( 255, 255, 255, 255 );
const Rgba DEFAULT_VERTEX_HIGHLIGHT_COLOR_FG( 0, 0, 0, 255 );
const Rgba DEFAULT_VERTEX_HIGHLIGHT_COLOR_BG( 255, 255, 0, 255 );
const Rgba DEFAULT_VERTEX_SELECTION_COLOR_FG( 0, 0, 0, 255 );
const Rgba DEFAULT_VERTEX_SELECTION_COLOR_BG( 255, 0, 0, 255 );
const Rgba DEFAULT_VERTEX_HIGHLIGHT_SELECTION_COLOR_FG( 0, 0, 0, 255 );
const Rgba DEFAULT_VERTEX_HIGHLIGHT_SELECTION_COLOR_BG( 255, 128, 0, 255 );
const Rgba DEFAULT_OBJECT_COLOR( 128, 0, 128, 255 );
const Rgba DEFAULT_OBJECT_HIGHLIGHT_COLOR_FG( 128, 0, 128, 255 );
const Rgba DEFAULT_OBJECT_HIGHLIGHT_COLOR_BG( 255, 255, 0, 255 );
const Rgba DEFAULT_OBJECT_SELECTION_COLOR_FG( 128, 0, 128, 255 );
const Rgba DEFAULT_OBJECT_SELECTION_COLOR_BG( 255, 0, 0, 255 );
const Rgba DEFAULT_OBJECT_HIGHLIGHT_SELECTION_COLOR_FG( 128, 0, 128, 255 );
const Rgba DEFAULT_OBJECT_HIGHLIGHT_SELECTION_COLOR_BG( 255, 128, 0, 255 );
const Rgba DEFAULT_ICON_COLOR( 64, 255, 255, 255 );


//-------------------------------------------------------------------------------------------------
// Static member initialization
//-------------------------------------------------------------------------------------------------
STATIC Options*	Options::s_singletonPtr = NULL;


//-------------------------------------------------------------------------------------------------
// Destructor()
//-------------------------------------------------------------------------------------------------
Options::~Options()
{
}


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
Options::Options( const char* optionsFilePath )
:
m_filePath( optionsFilePath ),
m_debugDrawHallOfMirrorsCheck( false ),
m_debugDrawMouseCursorRing( false ),
m_xyScrollSpeed( 0.5f ),
m_xyScrollInvertedX( false ),
m_xyScrollInvertedY( false ),
m_minVertexSelectWorldDist( DEFAULT_MIN_VERTEX_SELECT_WORLD_DIST ),
m_maxXYSelectScreenDist( DEFAULT_MAX_XY_SELECT_SCREEN_DIST ),
m_gridLineColor( DEFAULT_GRID_LINE_COLOR ),
m_gridDotColor( DEFAULT_GRID_DOT_COLOR ),
m_linedefColor( DEFAULT_LINEDEF_COLOR ),
m_linedefHighlightColorFg( DEFAULT_LINEDEF_HIGHLIGHT_COLOR_FG ),
m_linedefHighlightColorBg( DEFAULT_LINEDEF_HIGHLIGHT_COLOR_BG ),
m_linedefSelectionColorFg( DEFAULT_LINEDEF_SELECTION_COLOR_FG ),
m_linedefSelectionColorBg( DEFAULT_LINEDEF_SELECTION_COLOR_BG ),
m_linedefHighlightSelectionColorFg( DEFAULT_LINEDEF_HIGHLIGHT_SELECTION_COLOR_FG ),
m_linedefHighlightSelectionColorBg( DEFAULT_LINEDEF_HIGHLIGHT_SELECTION_COLOR_BG ),
m_protoLinedefColor( DEFAULT_PROTO_LINEDEF_COLOR ),
m_xyClearColor( DEFAULT_XY_CLEAR_COLOR ),
m_closestGridPointColor( DEFAULT_CLOSEST_GRID_POINT_COLOR ),
m_vertexColor( DEFAULT_VERTEX_COLOR ),
m_vertexHighlightColorFg( DEFAULT_VERTEX_HIGHLIGHT_COLOR_FG ),
m_vertexHighlightColorBg( DEFAULT_VERTEX_HIGHLIGHT_COLOR_BG ),
m_vertexSelectionColorFg( DEFAULT_VERTEX_SELECTION_COLOR_FG ),
m_vertexSelectionColorBg( DEFAULT_VERTEX_SELECTION_COLOR_BG ),
m_vertexHighlightSelectionColorFg( DEFAULT_VERTEX_HIGHLIGHT_SELECTION_COLOR_FG ),
m_vertexHighlightSelectionColorBg( DEFAULT_VERTEX_HIGHLIGHT_SELECTION_COLOR_BG ),
m_objectColor( DEFAULT_OBJECT_COLOR ),
m_objectHighlightColorFg( DEFAULT_OBJECT_HIGHLIGHT_COLOR_FG ),
m_objectHighlightColorBg( DEFAULT_OBJECT_HIGHLIGHT_COLOR_BG ),
m_objectSelectionColorFg( DEFAULT_OBJECT_SELECTION_COLOR_FG ),
m_objectSelectionColorBg( DEFAULT_OBJECT_SELECTION_COLOR_BG ),
m_objectHighlightSelectionColorFg( DEFAULT_OBJECT_HIGHLIGHT_SELECTION_COLOR_FG ),
m_objectHighlightSelectionColorBg( DEFAULT_OBJECT_HIGHLIGHT_SELECTION_COLOR_BG ),
m_iconColor( DEFAULT_ICON_COLOR )
{
	s_singletonPtr = this;
	SetToDefaults();

	/// See if the preferences file exists
	struct _stat results;
	int error = _stat( m_filePath.c_str(), &results );
	if( error )
	{
		theConsole.Printf( "Couldn't find preferences file \"%s\"; using defaults\n", m_filePath.c_str() );
		return;
	}

	/// Load and parse preferences file
	theConsole.Printf( "Reading editor preferences from \"%s\"...\n", m_filePath.c_str() );
	ParseFile optionsFile( m_filePath );
}


//-------------------------------------------------------------------------------------------------
// SetToDefaults
//-------------------------------------------------------------------------------------------------
void Options::SetToDefaults()
{
}


