//-------------------------------------------------------------------------------------------------
// Pane.hpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_Pane_
#define _include_Pane_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "Grid.hpp"
#include "ViewBase.hpp"
#include "gameCommon.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		Pane
// 
///////////////////////////////////////////////////////////////////////////////////////////////////
class Pane
{
private:
public:
	enum PaneType
	{
		PANE_LEAF,
		PANE_SPLIT_VERTICAL_CUT,
		PANE_SPLIT_HORIZONTAL_CUT,
	};

	static Pane*	s_focusPane;	// NULL if no pane is currently in focus

	PaneType	m_type;
	Pane*		m_frontPane;		// NULL if <m_type> is PANE_LEAF
	Pane*		m_backPane;			// NULL if <m_type> is PANE_LEAF
	ViewBase*	m_view;				// NULL if <m_type> is not PANE_LEAF
	float		m_splitFraction;	// Where (either vertically or horizontally) the split occurs, fractionally
	IntVector2	m_splitPoint;		// Where the split occurs, as far as actual (relative) integral screen coordinate
	IntVector2	m_mins;				// Minimum integral screen coordinate of entire panespace
	IntVector2	m_maxs;				// Maximum integral screen coordinate of entire panespace (inclusive)
	bool		m_hasFocus;			// when true, the pane has focus (cursor is/was here, can process input, etc.)

	/// Constructors / destructors
	Pane( ViewBase* view, const IntVector2& paneMins, const IntVector2& paneMaxs );
	~Pane(); // destructor

	void UpdateFocus( const IntVector2& cursorPos );
	IntVector2 GetPaneSize() const;
	void Draw();
	void Update();
	void DrawPaneBorder();
	bool ProcessInputEvent( TheInputSystem::inputEventType_e eventType, byte_t key );
	void Split( ViewBase* newView, float splitFraction, bool isHorizontal, bool isNewPaneFront );
	void SplitRight( ViewBase* newView, float splitFraction );
	void SplitLeft( ViewBase* newView, float splitFraction );
	void SplitAbove( ViewBase* newView, float splitFraction );
	void SplitBelow( ViewBase* newView, float splitFraction );
};


#endif // _include_Pane_
