//-------------------------------------------------------------------------------------------------
// TheEditor.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "TheEditor.hpp"
#include "TheGame.hpp" // hax


//-------------------------------------------------------------------------------------------------
// Globals
//-------------------------------------------------------------------------------------------------
TheEditor* theEditor = NULL;


//-------------------------------------------------------------------------------------------------
// Destructor()
//-------------------------------------------------------------------------------------------------
TheEditor::~TheEditor()
{
}


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
TheEditor::TheEditor()
:
m_rootPane( NULL ),
m_options( NULL ),
m_isEditorActive( false ),
m_underlay( NULL ),
m_underlayBrightness( 0.5f )
{
}


//-------------------------------------------------------------------------------------------------
// RunFrame
//-------------------------------------------------------------------------------------------------
void TheEditor::RunFrame()
{
	if( !m_isEditorActive )
	{
		return;
	}

	SetFocus();
	UpdateEverything();
	DrawEverything();
	theRenderer.ResetViewport();
}


//-------------------------------------------------------------------------------------------------
// PreInit
//-------------------------------------------------------------------------------------------------
void TheEditor::PreInit()
{
	OpenStartupDialog();
	m_options = new Options( "data/BallsToTheWall.ini" );

	if( !m_isEditorActive )
	{
		return;
	}

}


//-------------------------------------------------------------------------------------------------
// PostInit
//-------------------------------------------------------------------------------------------------
void TheEditor::PostInit()
{
	if( !m_isEditorActive )
	{
		return;
	}

	g_currentArea = new MapArea;
//	g_currentArea = new MapArea( "data/TestMap.txt" );

	m_rootPane = new Pane( new ViewXY, IntVector2::ONE, theVideoSystem.GetCurrentVidModeSize() - (2 * IntVector2::ONE) );
	m_rootPane->SplitRight( new ViewInfo, 0.75f );
//	m_rootPane->SplitRight( new ViewXY, 0.60f );
//	m_rootPane->m_backPane->SplitBelow( new ViewInfo, 0.50f );
//	m_rootPane->m_frontPane->SplitBelow( new ViewConsole, 0.70f );
//	m_rootPane->m_frontPane->SplitLeft( new ViewXY, 0.20f );
//	m_rootPane->m_frontPane->m_backPane->SplitBelow( new View3d, 0.70f );
//	m_rootPane->m_frontPane->m_backPane->m_frontPane->SplitAbove( new ViewXY, 0.20f );
}


//-------------------------------------------------------------------------------------------------
// UpdateEverything
//-------------------------------------------------------------------------------------------------
void TheEditor::UpdateEverything()
{
	if( !m_isEditorActive )
	{
		return;
	}

	if( m_rootPane )
	{
		m_rootPane->Update();
	}
}


//-------------------------------------------------------------------------------------------------
// Sets the background image
//-------------------------------------------------------------------------------------------------
void TheEditor::SetBackgroundImage( const std::string& backgroundImage )
{
	LoadUnderlayTexture( backgroundImage.c_str() );
}


//-------------------------------------------------------------------------------------------------
// LoadUnderlayTexture
//-------------------------------------------------------------------------------------------------
void TheEditor::LoadUnderlayTexture( const char* texturePath )
{
	if( m_underlay )
	{
		delete m_underlay;
		m_underlay = NULL;
	}

	if( texturePath )
	{
		m_underlay = new Texture( texturePath );
	}
}


//-------------------------------------------------------------------------------------------------
// StartEditing
//-------------------------------------------------------------------------------------------------
void TheEditor::StartEditing()
{
	theEditor->m_isEditorActive = true;
	if( g_currentArea )
	{
		g_currentArea->DeleteAllElements();
		delete g_currentArea;
		g_currentArea = NULL;
	}

	if( m_rootPane )
	{
		m_rootPane = NULL;
		/// FIXME: recursively free the root / view tree
	}

	PostInit();
}



//-------------------------------------------------------------------------------------------------
// StopEditing
//-------------------------------------------------------------------------------------------------
void TheEditor::StopEditing()
{
	theEditor->m_isEditorActive = false;
	if( g_currentArea )
	{
		g_currentArea->DeleteAllElements();
		delete g_currentArea;
		g_currentArea = NULL;
	}

	if( m_rootPane )
	{
		m_rootPane = NULL;
		/// FIXME: recursively free the root / view tree
	}
}



//-------------------------------------------------------------------------------------------------
// PlayCurrentMap
//-------------------------------------------------------------------------------------------------
void TheEditor::PlayCurrentMap()
{
	theGame->ForceConsoleState( false );
	theGame->SetGameState( TheGame::STATE_STAGING );

	std::string mapName;
	if( g_currentArea )
	{
		mapName = g_currentArea->m_mapName;
		StopEditing();
		theGame->WarpTo( mapName );
	}
	else
	{
		StopEditing();
	}
}


//-------------------------------------------------------------------------------------------------
// EditCurrentMap
//-------------------------------------------------------------------------------------------------
void TheEditor::EditCurrentMap()
{
	theGame->ForceConsoleState( false );
	theGame->SetGameState( TheGame::STATE_EDITING );
	m_isEditorActive = true;
	PostInit();

	std::string& mapName = theGame->GetCurrentMapName();
	if( mapName.length() > 0 )
	{
		LoadMap( mapName );

		std::string	backgroundPath( "data/maps/" );
		backgroundPath	+= mapName;
		backgroundPath	+= ".tga" ;
		SetBackgroundImage( backgroundPath );
	}
}

