//-------------------------------------------------------------------------------------------------
// TheEditor_Input.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "TheEditor.hpp"



//-------------------------------------------------------------------------------------------------
// ProcessInputEvent
//-------------------------------------------------------------------------------------------------
bool TheEditor::ProcessInputEvent( TheInputSystem::inputEventType_e eventType, byte_t key )
{
	bool wasHandled = false;

	if( ViewBase::s_focusView )
	{
		wasHandled = ViewBase::s_focusView->ProcessInputEvent( eventType, key );
	}

	if( !wasHandled && Pane::s_focusPane )
	{
		wasHandled = Pane::s_focusPane->ProcessInputEvent( eventType, key );
	}

	return wasHandled;
}


//-------------------------------------------------------------------------------------------------
// SetFocus
//-------------------------------------------------------------------------------------------------
void TheEditor::SetFocus()
{
	/// Get the cursor position from the input system
	IntVector2 mousePos = theInputSystem.GetMousePosClientAbsolute();
	Pane::s_focusPane = NULL;
	ViewBase::s_focusView = NULL;
	if( m_rootPane )
	{
		m_rootPane->UpdateFocus( mousePos );
	}
}


//-------------------------------------------------------------------------------------------------
// SetPropertyForHighlightedElements
//-------------------------------------------------------------------------------------------------
void TheEditor::SetPropertyForHighlightedElements( const std::string& key, const std::string& value )
{
	g_currentArea->SetPropertyForHighlightedElements( key, value );
}


//-------------------------------------------------------------------------------------------------
// DeletePropertyForHighlightedElements
//-------------------------------------------------------------------------------------------------
void TheEditor::DeletePropertyForHighlightedElements( const std::string& key )
{
	g_currentArea->DeletePropertyForHighlightedElements( key );
}


//-------------------------------------------------------------------------------------------------
// LoadMap
//-------------------------------------------------------------------------------------------------
void TheEditor::LoadMap( const std::string& mapName )
{
	std::string mapPath( "data/maps/" );
	mapPath += mapName;
	mapPath += ".map";

	g_currentArea->DeleteAllElements();
	g_currentArea->PopulateFromFile( mapPath );
	g_currentArea->m_mapName = mapName;
}


//-------------------------------------------------------------------------------------------------
// SaveMap
//-------------------------------------------------------------------------------------------------
void TheEditor::SaveMap( const std::string& mapName )
{
	std::string mapPath( "data/maps/" );
	mapPath += mapName;
	mapPath += ".map";

	g_currentArea->SaveToFile( mapPath );
}



