//-------------------------------------------------------------------------------------------------
// TheEditor.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "TheEditor.hpp"
#include "resource.h"
#include "TheGame.hpp"

enum
{
	DIALOG_RESULT_CANCEL = 1,
	DIALOG_RESULT_OK = 2,
};


//-------------------------------------------------------------------------------------------------
// SetVideoModeFromStartupDialog
//-------------------------------------------------------------------------------------------------
void SetVideoModeFromStartupDialog( HWND hWnd )
{
	IntVector2 screenRes( 1280, 1024 );
	unsigned int flags = 0;

	if( IsDlgButtonChecked( hWnd, IDC_RADIO_1280_1024_FULLSCREEN_EDITOR ) )
	{
		screenRes.SetXY( 1280, 1024 );
		flags |= TheVideoSystem::VIDMODE_FLAG_FULLSCREEN;
		theGame->SetGameState( TheGame::STATE_EDITING );
		theEditor->m_isEditorActive = true;
	}
	else if( IsDlgButtonChecked( hWnd, IDC_RADIO_1280_1024_WINDOWED_EDITOR ) )
	{
		screenRes.SetXY( 1280, 1024 );
		theGame->SetGameState( TheGame::STATE_EDITING );
		theEditor->m_isEditorActive = true;
	}
	else if( IsDlgButtonChecked( hWnd, IDC_RADIO_1280_1024_FULLSCREEN_GAME ) )
	{
		screenRes.SetXY( 1280, 1024 );
		flags |= TheVideoSystem::VIDMODE_FLAG_FULLSCREEN;
		theGame->SetGameState( TheGame::STATE_STARTUP );
		theEditor->m_isEditorActive = false;
	}
	else if( IsDlgButtonChecked( hWnd, IDC_RADIO_1280_1024_WINDOWED_GAME ) )
	{
		screenRes.SetXY( 1280, 1024 );
		theGame->SetGameState ( TheGame::STATE_STARTUP );
		theEditor->m_isEditorActive = false;
	}

	theVideoSystem.SetDefaultVidModeProperties( screenRes.x, screenRes.y, 32, 85, flags );
}




//-------------------------------------------------------------------------------------------------
// StartupDialogOnOkay
//-------------------------------------------------------------------------------------------------
void StartupDialogOnOkay( HWND hWnd )
{
	theConsole.Printf( "ON OKAY\n" );
	SetVideoModeFromStartupDialog( hWnd );
	EndDialog( hWnd, DIALOG_RESULT_OK );
}


//-------------------------------------------------------------------------------------------------
// StartupDialogOnCancel
//-------------------------------------------------------------------------------------------------
void StartupDialogOnCancel( HWND hWnd )
{
	theConsole.Printf( "ON CANCEL\n" );
	EndDialog( hWnd, DIALOG_RESULT_CANCEL );
}


//-------------------------------------------------------------------------------------------------
// StartupDialogOnHelp
//-------------------------------------------------------------------------------------------------
void StartupDialogOnHelp( HWND hWnd )
{
	hWnd; // FIXME: placate the compiler

	theConsole.Printf( "ON HELP\n" );
//	EndDialog( hWnd, DIALOG_RESULT_CANCEL );
}



//-------------------------------------------------------------------------------------------------
// StartupDialogProc
//-------------------------------------------------------------------------------------------------
INT_PTR CALLBACK StartupDialogProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
//	theConsole.Printf( "StartupDialogProc was called, %d, %d, %d", msg, wParam, lParam );
	lParam; // FIXME: placate the compiler

	if( msg == WM_INITDIALOG )
	{
		CheckRadioButton( hWnd, IDC_RADIO_1280_1024_FULLSCREEN_EDITOR, IDC_RADIO_1280_1024_WINDOWED_GAME, IDC_RADIO_1280_1024_WINDOWED_GAME );
	}
	else if( msg == WM_COMMAND )
	{
		switch( LOWORD( wParam ) )
		{
			case IDOK:		StartupDialogOnOkay( hWnd );		break;
			case IDCANCEL:	StartupDialogOnCancel( hWnd );		break;
			case IDHELP:	StartupDialogOnHelp( hWnd );		break;
		}
	}

	return (INT_PTR) 0;
}


//-------------------------------------------------------------------------------------------------
// OpenStartupDialog
//-------------------------------------------------------------------------------------------------
void TheEditor::OpenStartupDialog()
{
	/// Create a dialog
	INT_PTR result = DialogBox(
		theApplication.GetHInstance(),
		MAKEINTRESOURCE( IDD_DIALOG_STARTUP ),
		theVideoSystem.GetHWnd(),
		StartupDialogProc );

	theConsole.Printf( "Dialog returned; return-value was %d\n", (int)result );
	if( result == (INT_PTR) DIALOG_RESULT_CANCEL )
	{
		theConsole.Printf( "Dialog returned : CANCEL\n" );
		theApplication.RequestShutdown();
	}
	else if( result == (INT_PTR) DIALOG_RESULT_OK )
	{
		theConsole.Printf( "Dialog returned : OK\n" );
	}
}


