//-------------------------------------------------------------------------------------------------
// TheGame.hpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_TheGame_
#define _include_TheGame_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "gameCommon.hpp"
#include "Map.hpp"
#include "Ball.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		TheGame
// 
///////////////////////////////////////////////////////////////////////////////////////////////////
class TheGame
{
public:

	enum GameState
	{
		STATE_EDITING,
		STATE_STAGING,
		STATE_RUNNING,
		STATE_STARTUP,
		STATE_MAP_COMPLETE_SUCCESS,
	};

	enum Sounds
	{
		SOUND_ITEM_COLLECTED,
		SOUND_BALL_SHOT,
		SOUND_BALL_BOUNCE,
		SOUND_MAP_COMPLETE_SUCCESS,
	};

private:

	std::vector< Ball* >	_balls;
	std::vector< Map* >		_maps;
	unsigned int			_currentMap;

	GameState				_state ;
	double					_timeEnteredState;

	bool					_commandConsoleEnabled ;
	char					_commandBuffer[ 255 ] ;
	int						_commandBufferPosition ;
	Texture*				_titleScreen ;
	Texture*				_gameUIScreen ;
	Texture*				_directionalArrow ;
	Texture*				_mapCompleteSuccessScreen;

	int						_initialRotationAngle ;
	int						_initialPower ;
	int						_updatePowerDelta ;
	int						_updateRotationDelta ;
	bool					_forceRotation;
	bool					_forcePower;
	bool					_isTurboRotation;
	bool					_isTurboPower;
	double					_nextRotationUpdateTime ;
	double					_nextPowerUpdateTime ;
	Ball*					_theBall ;
	std::string				_currentMapName ;
	bool					_isSoundEnabled;

public:

	/// Constructors / destructors
	TheGame(); // default constructor
	~TheGame(); // destructor

	void			SetGameState( GameState state );
	GameState		GetGameState( void )						{ return _state ; }
	double			ElapsedTimeInCurrentState();
	std::string&	GetCurrentMapName() { return _currentMapName; };


	/// from TheGame.cpp
	void PreInit();
	void PostInit();
	void RunFrame();
	void RunFrameStartup( void );
	void RunFrameStaging( void );
	void RunFrameRunning( void );
	void RunFrameEditing( void );
	void RunFrameMapCompleteSuccess( void );

	void ProcessCommandLine( const std::string& cmdLine );
	bool ProcessInputEvent( TheInputSystem::inputEventType_e eventType, byte_t key );
	bool ProcessGameInput( TheInputSystem::inputEventType_e eventType, byte_t key );

	bool OnKeyDown( const byte_t key );
	bool ExecuteCommand( void );

	/// from TheGame_Update.cpp
	void UpdateEverything();
	void UpdateInitialStaging( void );


	/// from TheGame_Draw.cpp
	void SetUpCoordinateSystem();
	void DrawEverything();
	void DrawUI( void );
	void DrawConsole( void );


	void WarpTo( const std::string& mapName );
	void WarpToNextMap();
	int FindLoadedMap( const std::string& tgaFilename);
	bool DoesMapExist( const std::string& mapFilename );
	int LoadMap( const std::string& mapName );
	Map& GetCurrentMap() const { return *_maps[ _currentMap ]; }

	bool IsMapComplete() const;

	void AddBall( Vector2 position );
	void ShootBall( int power, int rotationAngle );
	void CollectItem( void );
	void BallBounce( float impactHardness );

	void ForceConsoleState( bool open );
};


//-------------------------------------------------------------------------------------------------
// Externally viewable globals
//-------------------------------------------------------------------------------------------------
extern TheGame* theGame;
extern std::list<Vector2> g_TEMP_lastPath;
extern double g_TEMP_pathSecondsLeft;


#endif // _include_TheGame_
