//-------------------------------------------------------------------------------------------------
// TheGame_Draw.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "TheGame.hpp"



//-------------------------------------------------------------------------------------------------
// SetUpCoordinateSystem
// 
// Set up the drawing context
//-------------------------------------------------------------------------------------------------
void TheGame::SetUpCoordinateSystem()
{
	theRenderer.SetCoordinateSystemToScreenCoordinates2d();
}


//-------------------------------------------------------------------------------------------------
// DrawEverything
// 
// Render the scene
//-------------------------------------------------------------------------------------------------
void TheGame::DrawEverything()
{
	theRenderer.ResetViewportAndScissor();
	theRenderer.SetCoordinateSystemToScreenCoordinates2d();
	theRenderer.ClearScreen( Rgba::BLACK );
	
	if( _maps.size() > 0 && _maps.size() > _currentMap )
	{
		_maps[ _currentMap ]->Draw();
	}

	DrawUI();

	if( _balls.size() > 0 )
	{
		if ( GetGameState() == STATE_STAGING )
		{
			glBlendFunc( GL_SRC_ALPHA, GL_ONE );
			glEnable( GL_BLEND );
			theRenderer.DrawTexturedRectCentered2d( *_directionalArrow, _balls[0]->m_pos, Vector2(128.0f,128.0f), (float)_initialRotationAngle );
		}

		glLineWidth( 2.0f );
		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		glEnable( GL_BLEND );
		int iterCount = 0;
		int numVerts = g_TEMP_lastPath.size();
		if( numVerts >= 2 )
		{
			glBegin( GL_LINE_STRIP );
			{
				std::list<Vector2>::iterator begin = g_TEMP_lastPath.begin();
				std::list<Vector2>::iterator end = g_TEMP_lastPath.end();
				std::list<Vector2>::iterator iter;

				for( iter = begin; iter != end; ++ iter )
				{
					int countShortOfFresh = numVerts - iterCount;
					float fraction = (float)countShortOfFresh / 150.0f;
					float fraction2 = (float)countShortOfFresh / 400.0f;
					fraction = ClampFloat( fraction, 0.0f, 1.0f );
					fraction2 = ClampFloat( fraction2, 0.0f, 1.0f );
					iterCount ++;

					glColor4f( 1.0f - 0.5f * fraction2, 1.0f - 0.8f * fraction, 1.0f, 1.0f - 0.6f * fraction2 );
					Vector2& point = (*iter);
					glVertex2f( point.x, point.y );
				}
			}
			glEnd(); // GL_LINE_STRIP
			glLineWidth( 1.0f );
		}

		_balls[0]->Draw();
	}

	DrawConsole();
}



///--------------------------------------------------------------------------------------------------
/// Draws the UI Screen
///--------------------------------------------------------------------------------------------------
void TheGame::DrawUI( void )
{
	glDisable( GL_BLEND );

	if ( _gameUIScreen )
	{
		theRenderer.SetCoordinateSystemToScreenCoordinates2d();
		theRenderer.DrawTexturedRect2d( *_gameUIScreen, Vector2( 1024.0f, 0.0f ), Vector2( 1280.0f, 1024.0f ) );
	}

	// Draw the Power Bar
	float powerFraction	= (float)_initialPower / 100.0f ;
	float red			= 255.0f * powerFraction ;
	float green			= 0.0f ;
	float blue			= 255.0f - red ;
	Rgba color( (unsigned char)red, (unsigned char)green, (unsigned char)blue ) ;
	Vector2	barStart( 1052.0f, 364.0f );
	Vector2 barEnd( 1052.0f + ((float)_initialPower * 2.0f), 404.0f );
	theRenderer.DrawAABB2d( barStart, barEnd, color, true );
	theRenderer.XYPrintf( 1048.0f + 90.0f, 384.0f, "%d", _initialPower );

	// Draw the Angle meter
	theRenderer.XYPrintf( 1048.0f + 90.0f, 478.0f, "%d", _initialRotationAngle );
}


///--------------------------------------------------------------------------------------------------
/// Draws the console box if enabled
///--------------------------------------------------------------------------------------------------
void TheGame::DrawConsole( void )
{
	theRenderer.SetCoordinateSystemToScreenCoordinates2d();
	theRenderer.ResetViewportAndScissor();

	bool isLeftJustified = true;

	if ( _commandConsoleEnabled )
	{
		Rgba	backgroundColor( 0, 0, 127, 127 );
		float	centerX	= 256.0f + 768.0f/2.0f ;
		float	centerY	= 768.0f/2.0f;
		float	width	= 200.0f ;
		float	height	= 24.0f ;
		float	left	= centerX - width ;
		float	right	= centerX + width ;
		float	top		= centerY - height ;
		float	bottom	= centerY + height ;

		if( isLeftJustified )
		{
			right -= left;
			left = 0.0f;
		}

		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		AABB2 bounds( left, top, right, bottom );
		theRenderer.DrawAABB2d( bounds, backgroundColor, true );
		theRenderer.DrawAABB2d( bounds, Rgba::WHITE, false );
		theRenderer.XYPrintf( left+20.0f, centerY+12.0f, "%s", _commandBuffer );
		theRenderer.XYPrintf( left+20.0f, centerY-6.0f, "Enter command; hit ENTER to submit, ESCAPE or ~ to cancel:" );
	}
}
