//-------------------------------------------------------------------------------------------------
// ViewBase.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "ViewBase.hpp"


//-------------------------------------------------------------------------------------------------
// Static member initialization
//-------------------------------------------------------------------------------------------------
STATIC ViewBase* ViewBase::s_focusView = NULL;


//-------------------------------------------------------------------------------------------------
// Destructor()
//-------------------------------------------------------------------------------------------------
ViewBase::~ViewBase()
{
}


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
ViewBase::ViewBase()
:
m_isDirty( true ),
m_clearColor( Rgba( 64, 64, 64 ) ),
m_hasFocus( false )
{
	theConsole.Printf( "Constructing ViewBase #%u\n", (unsigned int) this );
}


//-------------------------------------------------------------------------------------------------
// Draw
//-------------------------------------------------------------------------------------------------
VIRTUAL void ViewBase::Draw()
{
	/// Do nothing if window does not require redrawing
	if( !m_isDirty )
	{
		return;
	}

	theRenderer.SetCoordinateSystem2d( Vector2::ZERO, Vector2::ONE );
	theRenderer.ClearScreen( m_clearColor * 0.5f );
}


//-------------------------------------------------------------------------------------------------
// Update
//-------------------------------------------------------------------------------------------------
VIRTUAL void ViewBase::Update()
{
}


//-------------------------------------------------------------------------------------------------
// GetMousePosViewClientAbsoluteFloat
//-------------------------------------------------------------------------------------------------
const Vector2 ViewBase::GetMousePosViewClientAbsoluteFloat() const
{
	IntVector2 clientPos = theInputSystem.GetMousePosClientAbsolute();
	return Vector2( (float)(clientPos.x - m_mins.x), (float)(clientPos.y - m_mins.y) );
}


//-------------------------------------------------------------------------------------------------
// GetMousePosViewClientAbsolute
//-------------------------------------------------------------------------------------------------
const IntVector2 ViewBase::GetMousePosViewClientAbsolute() const
{
	IntVector2 clientPos = theInputSystem.GetMousePosClientAbsolute();
	return (clientPos - m_mins);
}


//-------------------------------------------------------------------------------------------------
// GetViewSize
//-------------------------------------------------------------------------------------------------
IntVector2 ViewBase::GetViewSize() const
{
	return (m_maxs - m_mins) + IntVector2::ONE;
}



//-------------------------------------------------------------------------------------------------
// ProcessInputEvent
//-------------------------------------------------------------------------------------------------
VIRTUAL bool ViewBase::ProcessInputEvent( TheInputSystem::inputEventType_e eventType, byte_t key )
{
	UNUSED_ARG eventType, key; // placate the compiler
	return false;
}



