//-------------------------------------------------------------------------------------------------
// ViewInfo.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "ViewInfo.hpp"


//-------------------------------------------------------------------------------------------------
// Destructor()
//-------------------------------------------------------------------------------------------------
ViewInfo::~ViewInfo()
{
}


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
ViewInfo::ViewInfo()
{
	theConsole.Printf( "Constructing ViewInfo #%u\n", (unsigned int) this );
	m_clearColor = Rgba::DARKBLUE;
}



//-------------------------------------------------------------------------------------------------
// Draw
//-------------------------------------------------------------------------------------------------
VIRTUAL void ViewInfo::Draw()
{
	/// Do nothing if window does not require redrawing
	if( !m_isDirty )
	{
		return;
	}

	Rgba clearColor = m_clearColor;
	clearColor *= 0.5f;
	theRenderer.ClearScreen( clearColor );

	_DrawText();
}


//-------------------------------------------------------------------------------------------------
// _DrawText
//-------------------------------------------------------------------------------------------------
void ViewInfo::_DrawText()
{
	theRenderer.PushCoordinateSystem();

	Vector2 topLeft( (float)m_mins.x, (float)m_mins.y );
	Vector2 bottomRight( (float)m_maxs.x, (float)m_maxs.y );
	Vector2 center = 0.5f * (topLeft + bottomRight);
	theRenderer.SetCoordinateSystem2d( topLeft, bottomRight );
	theRenderer.SetColor( Rgba::WHITE );
	const float fontAverageCharacterWidth = 5.0f; // FIXME: ask the renderer
	const float fontAverageCharacterHeight = 14.0f; // FIXME: ask the renderer
	const float	smallTabSize = 25.0f;
	const float	largeTabSize = 100.0f;

	Vector2 printPosition( topLeft.x + fontAverageCharacterWidth, topLeft.y + fontAverageCharacterHeight );
	Vector2 lineFeed( 0.0f, fontAverageCharacterHeight );
	Vector2 smallTab( smallTabSize, 0.0f );
	Vector2 largeTab( largeTabSize, 0.0f );

	theRenderer.XYPrintf( printPosition, "Info View" );
	printPosition += lineFeed;
	printPosition += lineFeed;

	/// Selected objects
	if( g_currentArea->NumSelectedElements() > 0 )
	{
		int numElements	= g_currentArea->NumSelectedElements();
		int numObjects	= g_currentArea->NumSelectedElementsOfType( MapElement::ELEMENT_TYPE_OBJECT );
		int numLinedefs	= g_currentArea->NumSelectedElementsOfType( MapElement::ELEMENT_TYPE_LINEDEF );
		int numVertexes	= g_currentArea->NumSelectedElementsOfType( MapElement::ELEMENT_TYPE_VERTEX );

		theRenderer.XYPrintf( printPosition, "%d Selected elements:", numElements );
		printPosition += lineFeed;

		if( numObjects > 0 )
		{
			theRenderer.XYPrintf( printPosition + smallTab, "%d Object(s)", numObjects );
			printPosition += lineFeed;
		}

		if( numLinedefs > 0 )
		{
			theRenderer.XYPrintf( printPosition + smallTab, "%d Linedef(s)", numLinedefs );
			printPosition += lineFeed;
		}

		if( numVertexes > 0 )
		{
			theRenderer.XYPrintf( printPosition + smallTab, "%d Vertex(es)", numVertexes );
			printPosition += lineFeed;
		}
	}
	else
	{
		theRenderer.XYPrintf( printPosition, "No selected elements" );
		printPosition += lineFeed;
	}
	printPosition += lineFeed;

	/// Highlighted object(s)
	if( g_currentArea->m_highlightList.size() > 0 )
	{
		MapElement* element = g_currentArea->m_highlightList.front();
		MapElement::ElementType type = element->m_type;
		if( type == MapElement::ELEMENT_TYPE_VERTEX )
		{
			MapVertex* vertex = reinterpret_cast< MapVertex* >( element );
			theRenderer.XYPrintf( printPosition, "Highlighted element:", 
				vertex->m_pos.x, vertex->m_pos.y );
			printPosition += lineFeed;

			theRenderer.XYPrintf( printPosition + smallTab, "Vertex at (%g, %g)",
				vertex->m_pos.x, vertex->m_pos.y );
			printPosition += lineFeed;
		}
		else if( type == MapElement::ELEMENT_TYPE_LINEDEF )
		{
			MapLinedef* linedef = reinterpret_cast< MapLinedef* >( element );
			theRenderer.XYPrintf( printPosition, "Highlighted element:" );
			printPosition += lineFeed;

			theRenderer.XYPrintf( printPosition + smallTab, "Linedef from (%g, %g) to (%g, %g)",
				linedef->m_start->m_pos.x,
				linedef->m_start->m_pos.y,
				linedef->m_end->m_pos.x,
				linedef->m_end->m_pos.y );
			printPosition += lineFeed;

			int numProperties = linedef->m_properties.GetNumKeyValues();
			if( numProperties > 0 )
			{
				printPosition += lineFeed;
				theRenderer.XYPrintf( printPosition, "Properties:" );
				printPosition += lineFeed;

				theRenderer.XYPrintf( printPosition + smallTab, "KEY" );
				theRenderer.XYPrintf( printPosition + smallTab + largeTab, "VALUE" );
				printPosition += lineFeed;

				theRenderer.XYPrintf( printPosition + smallTab, "~~~~~" );
				theRenderer.XYPrintf( printPosition + smallTab + largeTab, "~~~~~~~" );
				printPosition += lineFeed;

				MapProperties::KeyPairMap::iterator begin = linedef->m_properties.m_keyValuePairs.begin();
				MapProperties::KeyPairMap::iterator end = linedef->m_properties.m_keyValuePairs.end();
				MapProperties::KeyPairMap::iterator iter;
				for( iter = begin; iter != end; ++ iter )
				{
					std::string key = iter->first;
					std::string value = iter->second;

					theRenderer.XYPrintf( printPosition + smallTab, "%s", key.c_str() );
					theRenderer.XYPrintf( printPosition + smallTab + largeTab, "%s", value.c_str() );
					printPosition += lineFeed;
				}
			}
		}
		else if( type == MapElement::ELEMENT_TYPE_OBJECT )
		{
			MapObject* object = reinterpret_cast< MapObject* >( element );
			theRenderer.XYPrintf( printPosition, "Highlighted element:  Object at (%g, %g)",
				object->m_pos.x, object->m_pos.y );
			printPosition += lineFeed;

			int numProperties = object->m_properties.GetNumKeyValues();
			if( numProperties > 0 )
			{
				printPosition += lineFeed;
				theRenderer.XYPrintf( printPosition, "Properties:" );
				printPosition += lineFeed;

				theRenderer.XYPrintf( printPosition + smallTab, "KEY" );
				theRenderer.XYPrintf( printPosition + smallTab + largeTab, "VALUE" );
				printPosition += lineFeed;

				theRenderer.XYPrintf( printPosition + smallTab, "~~~~~" );
				theRenderer.XYPrintf( printPosition + smallTab + largeTab, "~~~~~~~" );
				printPosition += lineFeed;

				MapProperties::KeyPairMap::iterator begin = object->m_properties.m_keyValuePairs.begin();
				MapProperties::KeyPairMap::iterator end = object->m_properties.m_keyValuePairs.end();
				MapProperties::KeyPairMap::iterator iter;
				for( iter = begin; iter != end; ++ iter )
				{
					std::string key = iter->first;
					std::string value = iter->second;

					theRenderer.XYPrintf( printPosition + smallTab, "%s", key.c_str() );
					theRenderer.XYPrintf( printPosition + smallTab + largeTab, "%s", value.c_str() );
					printPosition += lineFeed;
				}
			}
		}
	}
	else
	{
		theRenderer.XYPrintf( printPosition, "No highlighted elements" );
		printPosition += lineFeed;
	}

	theRenderer.PopCoordinateSystem();
}


