//-------------------------------------------------------------------------------------------------
// ViewXY.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "ViewXY.hpp"
#include "TheEditor.hpp"


//-------------------------------------------------------------------------------------------------
// Static member initialization
//-------------------------------------------------------------------------------------------------
STATIC MapVertex*			ViewXY::m_newLineStart = NULL;
STATIC MapVertex*			ViewXY::m_newLineEnd = NULL;
STATIC ViewXY::EditMode		ViewXY::m_editMode = EDIT_MODE_VERTEX_LINE_OBJECT;


//-------------------------------------------------------------------------------------------------
// Destructor()
//-------------------------------------------------------------------------------------------------
ViewXY::~ViewXY()
{
}


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
ViewXY::ViewXY()
:
m_viewCenterWorldCoords( 512.0f, 512.0f ),
m_viewWorldMins( Vector2::ZERO ),
m_viewWorldMaxs( Vector2::ZERO ),
m_cursorGridCoordsOfLMBDragStart( Vector2::ZERO ),
m_cursorGridCoordsOfRMBDragStart( Vector2::ZERO ),
m_cursorGridCoordsOfLastSelectionDrag( Vector2::ZERO ),
m_scale( DEFAULT_SCALE_XY ),
m_grid( NULL ),
m_gridDrawMode( GRID_DRAW_LINES ),
m_inputState( INPUT_STATE_NORMAL ),
m_isPotentialLMBClick( false ),
m_isPotentialRMBClick( false ),
m_isLMBDrag( false ),
m_isRMBDrag( false ),
m_highlightedItemAtStartOfDrag( NULL ),
m_highlightedItemWasSelectedAtStartOfDrag( false )
{
	theConsole.Printf( "Constructing ViewXY #%u\n", (unsigned int) this );
	m_clearColor = Options::GetSingleton()->m_xyClearColor;
	m_grid = new Grid;
}


//-------------------------------------------------------------------------------------------------
// Draw
//-------------------------------------------------------------------------------------------------
VIRTUAL void ViewXY::Draw()
{
	/// Do nothing if window does not require redrawing
	if( !m_isDirty )
	{
		return;
	}

	Rgba clearColor = m_clearColor;
	clearColor *= 0.5f;
	theRenderer.ClearScreen( clearColor );

	_SetCoordinateSystem();
	_DrawUnderlay();
	_DrawGrid();
	_DrawWorld();
	_DrawText();
	_DrawIcons();
}


//-------------------------------------------------------------------------------------------------
// _UpdateHighlights
// 
// FIXME: this logic works, but is ass.  rewrite.
//-------------------------------------------------------------------------------------------------
void ViewXY::_UpdateHighlights()
{
	if( m_isLMBDrag && m_editMode == EDIT_MODE_VERTEX_LINE_OBJECT )
	{
		g_currentArea->ClearHighlight();
		if( m_highlightedItemAtStartOfDrag )
		{
			g_currentArea->HighlightElement( m_highlightedItemAtStartOfDrag );
		}
		return;
	}
//*
	float maxSelectDist = m_scale * Options::GetSingleton()->m_maxXYSelectScreenDist;
	float maxSelectDistSquared = maxSelectDist * maxSelectDist;
	float minVertexSelectDist = Options::GetSingleton()->m_minVertexSelectWorldDist;
	float minVertexSelectDistSquared = minVertexSelectDist * minVertexSelectDist;
	g_currentArea->ClearHighlight();
	Vector2 worldCoords = CalcWorldCoordsForCursor();

	/// Find the closest object
	float objectDistSquared = FLT_MAX;
	float vertexDistSquared = FLT_MAX;
	float linedefDistSquared = FLT_MAX;
	MapObject* object = g_currentArea->FindClosestMapObjectToWorldCoords( worldCoords, objectDistSquared );
	MapVertex* vertex = g_currentArea->FindClosestMapVertexToWorldCoords( worldCoords, vertexDistSquared );
	MapLinedef* linedef = g_currentArea->FindClosestMapLinedefToWorldCoords( worldCoords, linedefDistSquared );

	/// Auto-override object selection within a minimum distance
	if( object && objectDistSquared < minVertexSelectDistSquared )
	{
		g_currentArea->HighlightElement( object );
		return;
	}

	/// Auto-override vertex selection within a minimum distance
	if( vertex && vertexDistSquared < minVertexSelectDistSquared )
	{
		g_currentArea->HighlightElement( vertex );
		return;
	}

	/// Auto-reject anything too far away
	if( objectDistSquared > maxSelectDistSquared )
	{
		object = NULL;
	}
	if( vertexDistSquared > maxSelectDistSquared )
	{
		vertex = NULL;
	}
	if( linedefDistSquared > maxSelectDistSquared )
	{
		linedef = NULL;
	}

	/// Now look for the nearest object, period
	float closestDistSquared = MinFloat( objectDistSquared, linedefDistSquared, vertexDistSquared );
	if( object && closestDistSquared == objectDistSquared )
	{
		g_currentArea->HighlightElement( object );
		return;
	}
	else if( linedef && closestDistSquared == linedefDistSquared )
	{
		g_currentArea->HighlightElement( linedef );
		return;
	}
	else if( vertex && closestDistSquared == vertexDistSquared )
	{
		g_currentArea->HighlightElement( vertex );
		return;
	}
/*/
	float maxSelectDistance = m_scale * Options::GetSingleton()->m_maxXYSelectScreenDist;
	float maxSelectDistanceSquared = maxSelectDistance * maxSelectDistance;

	g_currentArea->ClearHighlight();
	Vector2 worldCoords = CalcWorldCoordsForCursor();

	bool forceVertexSelection = false;
	float closestVertexDistanceSquared = FLT_MAX;
	MapVertex* closestVertex = NULL;
	std::list<MapVertex*>::iterator vertex_begin = g_currentArea->m_vertexList.begin();
	std::list<MapVertex*>::iterator vertex_end = g_currentArea->m_vertexList.end();
	std::list<MapVertex*>::iterator vertex_iter;
	for( vertex_iter = vertex_begin; vertex_iter != vertex_end; ++ vertex_iter )
	{
		MapVertex* vertex = (*vertex_iter);
		float distSquared = DistanceSquaredBetweenPoints2d( vertex->m_pos, worldCoords );
		if( distSquared > maxSelectDistanceSquared )
		{
			continue;
		}

		if( distSquared < closestVertexDistanceSquared )
		{
			closestVertex = vertex;
			closestVertexDistanceSquared = distSquared;

			float minVertexSelectDist = Options::GetSingleton()->m_minVertexSelectWorldDist;
			float minVertexSelectDistSquared = minVertexSelectDist * minVertexSelectDist;
			if( distSquared < minVertexSelectDistSquared )
			{
				forceVertexSelection = true;
			}
		}
	}

	if( closestVertex && forceVertexSelection )
	{
		g_currentArea->HighlightElement( closestVertex );
		return; // if a vertex is highlighted, don't even look for a linedef to highlight
	}

	float closestLinedefDistanceSquared = FLT_MAX;
	MapLinedef* closestLinedef = NULL;
	std::list<MapLinedef*>::iterator begin = g_currentArea->m_linedefList.begin();
	std::list<MapLinedef*>::iterator end = g_currentArea->m_linedefList.end();
	std::list<MapLinedef*>::iterator iter;
	for( iter = begin; iter != end; ++ iter )
	{
		MapLinedef* linedef = (*iter);
		Vector2 displacement = linedef->m_end->m_pos - linedef->m_start->m_pos;
		LineSegment2 lineSegment( linedef->m_start->m_pos, displacement );
		float lineFraction;
		float distSquared = DistanceSquaredFromLineSegmentToPoint2d( lineSegment, worldCoords, &lineFraction );
		if( lineFraction == 0.0f || lineFraction == 1.0f || distSquared > maxSelectDistanceSquared )
		{
			/// If the linedef was closest only by virtue of its endpoints, ignore it (the vertex was closer!)
			/// (or if it was too far away)
			continue;
		}

		if( distSquared < closestLinedefDistanceSquared )
		{
			closestLinedef = linedef;
			closestLinedefDistanceSquared = distSquared;
		}
	}

	if( closestLinedef && closestVertex )
	{
		if( closestVertexDistanceSquared < closestLinedefDistanceSquared )
		{
			g_currentArea->HighlightElement( closestVertex );
			return;
		}
		else
		{
			g_currentArea->HighlightElement( closestLinedef );
			return;
		}
	}

	if( closestLinedef )
	{
		g_currentArea->HighlightElement( closestLinedef );
		return;
	}

	if( closestVertex )
	{
		g_currentArea->HighlightElement( closestVertex );
		return;
	}
//*/
}


//-------------------------------------------------------------------------------------------------
// _OnStartLMBDrag
//-------------------------------------------------------------------------------------------------
void ViewXY::_OnStartLMBDrag()
{
	m_isLMBDrag = true;
	m_cursorGridCoordsOfLMBDragStart = CalcGridCoordsForCursor();
	m_cursorGridCoordsOfLastSelectionDrag = m_cursorGridCoordsOfLMBDragStart;

	m_highlightedItemAtStartOfDrag = NULL;
	if( g_currentArea->m_highlightList.size() > 0 )
	{
		MapElement* highlightedElement = g_currentArea->m_highlightList.front();
		m_highlightedItemAtStartOfDrag = highlightedElement;
		if( highlightedElement->m_isSelected )
		{
			m_highlightedItemWasSelectedAtStartOfDrag = true;
		}
		else
		{
			m_highlightedItemWasSelectedAtStartOfDrag = false;
			g_currentArea->SelectElement( m_highlightedItemAtStartOfDrag );
		}
	}
}


//-------------------------------------------------------------------------------------------------
// _OnStartRMBDrag
//-------------------------------------------------------------------------------------------------
void ViewXY::_OnStartRMBDrag()
{
	m_isRMBDrag = true;
	m_cursorGridCoordsOfRMBDragStart = CalcGridCoordsForCursor();
}


//-------------------------------------------------------------------------------------------------
// _OnFinishLMBDrag
//-------------------------------------------------------------------------------------------------
void ViewXY::_OnFinishLMBDrag()
{
	if( m_highlightedItemAtStartOfDrag && !m_highlightedItemWasSelectedAtStartOfDrag )
	{
		g_currentArea->DeselectElement( m_highlightedItemAtStartOfDrag );
		m_highlightedItemAtStartOfDrag = NULL;
	}
}


//-------------------------------------------------------------------------------------------------
// _OnFinishRMBDrag
//-------------------------------------------------------------------------------------------------
void ViewXY::_OnFinishRMBDrag()
{
}


//-------------------------------------------------------------------------------------------------
// _UpdateZoom
//-------------------------------------------------------------------------------------------------
void ViewXY::_UpdateZoom()
{
	IntVector2 mouseScreenCoords = theInputSystem.GetMousePosClientAbsolute();
	Vector2 mouseWorldCoords = CalcWorldCoordsForScreenCoords( mouseScreenCoords );

	m_inputState = INPUT_STATE_ZOOMING;
	const float SCALE_UP = 1.01f;
	const float SCALE_DOWN = 1.0f / SCALE_UP;
	int mouseMovementY = theInputSystem.GetMouseDeltaY();
	while( mouseMovementY > 0 )
	{
		mouseMovementY --;
		m_scale *= SCALE_UP;
	}
	while( mouseMovementY < 0 )
	{
		mouseMovementY ++;
		m_scale *= SCALE_DOWN;
	}

	UpdateViewWorldMinsMaxs();

	/// Realign the view so the mouse cursor always maps to the same spot in world space
	AlignWorldCoordsToScreenCoords( mouseWorldCoords, mouseScreenCoords );
}


//-------------------------------------------------------------------------------------------------
// _UpdatePan
//-------------------------------------------------------------------------------------------------
void ViewXY::_UpdatePan()
{
	m_inputState = INPUT_STATE_PANNING;
	IntVector2 mouseMovement = theInputSystem.GetMouseDeltas();
	Vector2 worldMovement( -(float)mouseMovement.x, (float)mouseMovement.y );
	worldMovement *= Options::GetSingleton()->m_xyScrollSpeed;

	if( Options::GetSingleton()->m_xyScrollInvertedX )
	{
		worldMovement.x *= -1.0f;
	}
	if( Options::GetSingleton()->m_xyScrollInvertedY )
	{
		worldMovement.y *= -1.0f;
	}

	worldMovement *= m_scale;
	m_viewCenterWorldCoords += worldMovement;
	UpdateViewWorldMinsMaxs();
}


//-------------------------------------------------------------------------------------------------
// _UpdateModeAll
//-------------------------------------------------------------------------------------------------
void ViewXY::_UpdateModeAll()
{
	m_inputState = INPUT_STATE_NORMAL;

	UpdateViewWorldMinsMaxs();

	if( m_hasFocus )
	{
		/// Any mouse movement between RMB down and RMB up means drag, not click
		if( theInputSystem.GetMouseDeltaX() || theInputSystem.GetMouseDeltaY() )
		{
			m_isPotentialLMBClick = false;
			m_isPotentialRMBClick = false;

			if( theInputSystem.IsKeyDown( VK_LBUTTON ) )
			{
				if( !m_isLMBDrag )
				{
					_OnStartLMBDrag();
				}
			}

			if( theInputSystem.IsKeyDown( VK_RBUTTON ) )
			{
				if( !m_isRMBDrag )
				{
					_OnStartRMBDrag();
				}
			}
		}
/*
		/// Check for view scrolling, zooming, etc.
		if( theInputSystem.IsKeyDown( VK_RBUTTON ) )
		{
			if( theInputSystem.IsKeyDown( VK_SHIFT ) )
			{
				/// Using mouse to zoom
				_UpdateZoom();
			}
			else
			{
				/// Using mouse to scroll XY view
				_UpdatePan();
			}
		}
//*/
	}
	else
	{
		/// Clear drag flags if we don't have focus
		m_isLMBDrag = false;
		m_isRMBDrag = false;
	}
}


//-------------------------------------------------------------------------------------------------
// _UpdateSelectionDrag
//-------------------------------------------------------------------------------------------------
void ViewXY::_UpdateSelectionDrag()
{
	IntVector2 mouseDelta = theInputSystem.GetMouseDeltas();
	if( mouseDelta == IntVector2::ZERO )
	{
		return;
	}

	if( m_highlightedItemAtStartOfDrag && m_highlightedItemAtStartOfDrag->m_type == MapElement::ELEMENT_TYPE_VERTEX )
	{
		/// Snap the highlighted vertex to the nearest grid point, and move everything else with it accordingly
		MapVertex* vertex = reinterpret_cast<MapVertex*>( m_highlightedItemAtStartOfDrag );
		Vector2 mouseGridCoords = CalcGridCoordsForCursor();
		Vector2 selectionDragAmount = mouseGridCoords - vertex->m_pos;
		if( selectionDragAmount != Vector2::ZERO )
		{
			g_currentArea->TranslateSelection( selectionDragAmount );
			m_cursorGridCoordsOfLastSelectionDrag = mouseGridCoords;
		}
	}
	else
	{
		Vector2 mouseGridCoords = CalcGridCoordsForCursor();
		Vector2 selectionDragAmount = mouseGridCoords - m_cursorGridCoordsOfLastSelectionDrag;
		if( selectionDragAmount != Vector2::ZERO )
		{
			g_currentArea->TranslateSelection( selectionDragAmount );
			m_cursorGridCoordsOfLastSelectionDrag = mouseGridCoords;
		}
	}
}


//-------------------------------------------------------------------------------------------------
// _UpdateModeVertexLineObject
//-------------------------------------------------------------------------------------------------
void ViewXY::_UpdateModeVertexLineObject()
{
	if( m_hasFocus )
	{
		if( m_isLMBDrag )
		{
			_UpdateSelectionDrag();
		}
//*
		/// Check for view scrolling, zooming, etc.
		if( theInputSystem.IsKeyDown( VK_RBUTTON ) )
		{
			if( theInputSystem.IsKeyDown( VK_SHIFT ) )
			{
				/// Using mouse to zoom
				_UpdateZoom();
			}
			else
			{
				/// Using mouse to scroll XY view
				_UpdatePan();
			}
		}
//*/
		/// Assign highlight based on proximity
		_UpdateHighlights();
	}
}


//-------------------------------------------------------------------------------------------------
// _UpdateModeLineChainCreate
//-------------------------------------------------------------------------------------------------
void ViewXY::_UpdateModeLineChainCreate()
{
	if( m_hasFocus )
	{
//*
		/// Check for view scrolling, zooming, etc.
		if( theInputSystem.IsKeyDown( VK_RBUTTON ) )
		{
			if( theInputSystem.IsKeyDown( VK_SHIFT ) )
			{
				/// Using mouse to zoom
				_UpdateZoom();
			}
			else
			{
				/// Using mouse to scroll XY view
				_UpdatePan();
			}
		}
//*/
		Vector2 gridCoords = CalcGridCoordsForCursor();
		m_newLineEnd->m_pos = gridCoords;
	}
}


//-------------------------------------------------------------------------------------------------
// _UpdateModeEditElementProperties
//-------------------------------------------------------------------------------------------------
void ViewXY::_UpdateModeEditElementProperties()
{
	if( m_hasFocus )
	{
	}
}


//-------------------------------------------------------------------------------------------------
// Update
//-------------------------------------------------------------------------------------------------
VIRTUAL void ViewXY::Update()
{
	_UpdateModeAll();

	switch( m_editMode )
	{
		case EDIT_MODE_VERTEX_LINE_OBJECT:
			_UpdateModeVertexLineObject();
			break;
		case EDIT_MODE_LINE_CHAIN_CREATE:
			_UpdateModeLineChainCreate();
			break;
		case EDIT_MODE_EDIT_ELEMENT_PROPERTIES:
			_UpdateModeEditElementProperties();
	}
}


//-------------------------------------------------------------------------------------------------
// UpdateViewWorldMinsMaxs
//-------------------------------------------------------------------------------------------------
void ViewXY::UpdateViewWorldMinsMaxs()
{
	float inverseAspectRatio = 1.0f;
	IntVector2 viewSize = GetViewSize();
	if( viewSize.x > 0 )
	{
		inverseAspectRatio = (float) viewSize.y / (float) viewSize.x;
	}

	float numWorldUnitsWide = DEFAULT_UNSCALED_WORLD_WIDTH * m_scale;
	float numWorldUnitsHigh = numWorldUnitsWide * inverseAspectRatio;
	Vector2 halfViewWorldSize( 0.5f * numWorldUnitsWide, 0.5f * numWorldUnitsHigh );
	m_viewWorldMins = m_viewCenterWorldCoords - halfViewWorldSize;
	m_viewWorldMaxs = m_viewCenterWorldCoords + halfViewWorldSize;
}


//-------------------------------------------------------------------------------------------------
// _SetCoordinateSystem
//-------------------------------------------------------------------------------------------------
void ViewXY::_SetCoordinateSystem()
{
	IntVector2 viewSize = GetViewSize();
	if( viewSize.x < 1 || viewSize.y < 1 )
	{
		theRenderer.SetCoordinateSystem2d( Vector2::ZERO, Vector2::ONE );
		return;
	}

	Vector2 topLeft( m_viewWorldMins.x, m_viewWorldMaxs.y );
	Vector2 bottomRight( m_viewWorldMaxs.x, m_viewWorldMins.y );
	theRenderer.SetCoordinateSystem2d( topLeft, bottomRight );
}


//-------------------------------------------------------------------------------------------------
// _DrawUnderlay
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawUnderlay()
{
	if( !theEditor->m_underlay )
		return;

	Vector2 mins( 0.0f, 0.0f );
	Vector2 maxs( 1024.0f, 1024.0f );
	byte_t colorComponent = (byte_t)(255.0f * theEditor->m_underlayBrightness);
	Rgba color( colorComponent, colorComponent, colorComponent, 255 );
	theRenderer.DrawTexturedRect2d( *theEditor->m_underlay, mins, maxs, 0.0f, Vector2::ONE, color, true, false );
}


//-------------------------------------------------------------------------------------------------
// _DrawGrid
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawGrid()
{
	/// Highlight the closest grid point to the cursor
	Rgba closestPointColor = Options::GetSingleton()->m_closestGridPointColor;
	theRenderer.SetColor( closestPointColor );
	Vector2 gridCoords = CalcGridCoordsForCursor();

	glPointSize( 3.0f );
	glBegin( GL_POINTS );
	glVertex2f( gridCoords.x, gridCoords.y );
	glEnd(); // GL_POINTS

	if( m_gridDrawMode == GRID_DRAW_NONE )
	{
		return;
	}

	float gridSize = m_grid->m_gridSize;

	/// Starting at the mins, find the nearest grid x,y that is smaller
	Vector2 gridMins, gridMaxs;
	gridMins.x = ((float)(ceil(m_viewWorldMins.x / gridSize)) * gridSize) - gridSize;
	gridMins.y = ((float)(ceil(m_viewWorldMins.y / gridSize)) * gridSize) - gridSize;
	gridMaxs.x = ((float)(floor(m_viewWorldMaxs.x / gridSize)) * gridSize) + gridSize;
	gridMaxs.y = ((float)(floor(m_viewWorldMaxs.y / gridSize)) * gridSize) + gridSize;

	if( m_gridDrawMode == GRID_DRAW_LINES )
	{
		_DrawGridLines( gridMins, gridMaxs );
	}
	else if( m_gridDrawMode == GRID_DRAW_DOTS )
	{
		_DrawGridDots( gridMins, gridMaxs );
	}

	/// Draw dotted lines showing world extents for BallsToTheWall
//	if( !m_underlay )
	{
		glLineWidth( 1.0f );
		GLushort stipplePattern = 0x8888;
		glLineStipple( 3, stipplePattern );
//		glEnable( GL_LINE_STIPPLE );
		theRenderer.SetColor( Rgba::DARKCYAN );
		glBegin( GL_LINE_LOOP );
		{
			glVertex2f( 0.0f, 0.0f );
			glVertex2f( 1024.0f, 0.0f );
			glVertex2f( 1024.0f, 1024.0f );
			glVertex2f( 0.0f, 1024.0f );
		}
		glEnd(); // GL_LINE_LOOP
		glDisable( GL_LINE_STIPPLE );
	}
}


//-------------------------------------------------------------------------------------------------
// CalcPixelsPerGrid
//-------------------------------------------------------------------------------------------------
float ViewXY::CalcPixelsPerGrid()
{
	IntVector2 viewSize = GetViewSize();
	Vector2 viewWorldSize = m_viewWorldMaxs - m_viewWorldMins;
	float pixelsPerGridX = m_grid->m_gridSize * viewSize.x / viewWorldSize.x;
	return pixelsPerGridX;
}


//-------------------------------------------------------------------------------------------------
// _DrawGridLines
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawGridLines( const Vector2& gridMins, const Vector2& gridMaxs )
{
	float pixelsPerGridX = CalcPixelsPerGrid();

	Rgba gridColor = Options::GetSingleton()->m_gridLineColor;
	const float MIN_PIXELS_PER_GRID_LOW = 2.0f;
	const float MIN_PIXELS_PER_GRID_HIGH = 12.0f;

	/// Draw the origin
	theRenderer.DrawCircle2d( Vector2::ZERO, 8.0f, gridColor, 16 );

	if( pixelsPerGridX <= MIN_PIXELS_PER_GRID_LOW )
	{
		/// Don't draw the grid at all!
		return;
	}

	if( pixelsPerGridX > MIN_PIXELS_PER_GRID_LOW && pixelsPerGridX < MIN_PIXELS_PER_GRID_HIGH )
	{
		float fraction = (pixelsPerGridX - MIN_PIXELS_PER_GRID_LOW) / (MIN_PIXELS_PER_GRID_HIGH - MIN_PIXELS_PER_GRID_LOW);
		gridColor.a = (byte_t)( 255.0f * fraction );
	}

	/// Draw grid lines
	theRenderer.SetColor( gridColor );
	glLineWidth( 1.0f );
	glBegin( GL_LINES );

	/// Draw vertical lines
	for( float x = gridMins.x; x <= gridMaxs.x; x += m_grid->m_gridSize )
	{
		glVertex2f( x, gridMins.y );
		glVertex2f( x, gridMaxs.y );
	}
	/// Draw horizontal lines
	for( float y = gridMins.y; y <= gridMaxs.y; y += m_grid->m_gridSize )
	{
		glVertex2f( gridMins.x, y );
		glVertex2f( gridMaxs.x, y );
	}
	glEnd(); // GL_LINES
}


//-------------------------------------------------------------------------------------------------
// _DrawGridDots
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawGridDots( const Vector2& gridMins, const Vector2& gridMaxs )
{
	float pixelsPerGridX = CalcPixelsPerGrid();

	Rgba gridColor = Options::GetSingleton()->m_gridDotColor;
	const float MIN_PIXELS_PER_GRID_LOW = 2.0f;
	const float MIN_PIXELS_PER_GRID_HIGH = 12.0f;

	if( pixelsPerGridX <= MIN_PIXELS_PER_GRID_LOW )
	{
		/// Don't draw the grid at all!
		return;
	}

	if( pixelsPerGridX > MIN_PIXELS_PER_GRID_LOW && pixelsPerGridX < MIN_PIXELS_PER_GRID_HIGH )
	{
		float fraction = (pixelsPerGridX - MIN_PIXELS_PER_GRID_LOW) / (MIN_PIXELS_PER_GRID_HIGH - MIN_PIXELS_PER_GRID_LOW);
		gridColor.a = (byte_t)( 255.0f * fraction );
	}

	/// Draw grid dots
	theRenderer.SetColor( gridColor );
	glPointSize( 1.0f );
	glBegin( GL_POINTS );
	for( float x = gridMins.x; x <= gridMaxs.x; x += m_grid->m_gridSize )
	for( float y = gridMins.y; y <= gridMaxs.y; y += m_grid->m_gridSize )
	{
		glVertex2f( x, y );
	}
	glEnd(); // GL_POINTS
}


//-------------------------------------------------------------------------------------------------
// _DrawLinedef
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawLinedef( const MapVertex* start, const MapVertex* end, const Rgba& color, float thickness )
{
	glLineWidth( thickness );
	theRenderer.SetColor( color );
	glBegin( GL_LINES );
	glVertex2f( start->m_pos.x, start->m_pos.y );
	glVertex2f( end->m_pos.x, end->m_pos.y );
	glEnd(); // GL_LINES
}


//-------------------------------------------------------------------------------------------------
// _DrawLinedef
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawLinedef( const MapLinedef& linedef )
{
	bool isHighlighted = (linedef.m_highlightFrameNumber == theApplication.GetFrameNumber() );
	bool isSelected = linedef.m_isSelected;

	if( isHighlighted && isSelected )
	{
		_DrawLinedef( linedef.m_start, linedef.m_end, Options::GetSingleton()->m_linedefHighlightSelectionColorBg, 5.0f );
		_DrawLinedef( linedef.m_start, linedef.m_end, Options::GetSingleton()->m_linedefHighlightSelectionColorFg, 1.0f );
	}
	else if( isHighlighted )
	{
		_DrawLinedef( linedef.m_start, linedef.m_end, Options::GetSingleton()->m_linedefHighlightColorBg, 5.0f );
		_DrawLinedef( linedef.m_start, linedef.m_end, Options::GetSingleton()->m_linedefHighlightColorFg, 1.0f );
	}
	else if( isSelected )
	{
		_DrawLinedef( linedef.m_start, linedef.m_end, Options::GetSingleton()->m_linedefSelectionColorBg, 5.0f );
		_DrawLinedef( linedef.m_start, linedef.m_end, Options::GetSingleton()->m_linedefSelectionColorFg, 1.0f );
	}
	else
	{
		_DrawLinedef( linedef.m_start, linedef.m_end, Options::GetSingleton()->m_linedefColor, 1.0f );
	}
}


//-------------------------------------------------------------------------------------------------
// _DrawVertex
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawVertex( const MapVertex& vertex, const Rgba& color, float size )
{
	glPointSize( size );
	theRenderer.SetColor( color );
	glBegin( GL_POINTS );
	glVertex2f( vertex.m_pos.x, vertex.m_pos.y );
	glEnd(); // GL_POINTS
}


//-------------------------------------------------------------------------------------------------
// _DrawVertex
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawVertex( const MapVertex& vertex )
{
	bool isHighlighted = (vertex.m_highlightFrameNumber == theApplication.GetFrameNumber() );
	bool isSelected = vertex.m_isSelected;

	if( isHighlighted && isSelected )
	{
		_DrawVertex( vertex, Options::GetSingleton()->m_vertexHighlightSelectionColorBg, 7.0f );
		_DrawVertex( vertex, Options::GetSingleton()->m_vertexHighlightSelectionColorFg, 3.0f );
	}
	else if( isHighlighted )
	{
		_DrawVertex( vertex, Options::GetSingleton()->m_vertexHighlightColorBg, 7.0f );
		_DrawVertex( vertex, Options::GetSingleton()->m_vertexHighlightColorFg, 3.0f );
	}
	else if( isSelected )
	{
		_DrawVertex( vertex, Options::GetSingleton()->m_vertexSelectionColorBg, 7.0f );
		_DrawVertex( vertex, Options::GetSingleton()->m_vertexSelectionColorFg, 3.0f );
	}
	else
	{
		_DrawVertex( vertex, Options::GetSingleton()->m_vertexColor, 3.0f );
	}
}


//-------------------------------------------------------------------------------------------------
// _DrawObject
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawObject( const MapObject& object, const Rgba& color, float paddingSize )
{
	glPointSize( 20.0f + paddingSize ); // FIXME: use object's bounds for drawing
	theRenderer.SetColor( color );
	glBegin( GL_POINTS );
	glVertex2f( object.m_pos.x, object.m_pos.y );
	glEnd(); // GL_POINTS
}


//-------------------------------------------------------------------------------------------------
// _DrawObject
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawObject( const MapObject& object )
{
	bool isHighlighted = (object.m_highlightFrameNumber == theApplication.GetFrameNumber() );
	bool isSelected = object.m_isSelected;

	if( isHighlighted && isSelected )
	{
		_DrawObject( object, Options::GetSingleton()->m_objectHighlightSelectionColorBg, 2.0f );
		_DrawObject( object, Options::GetSingleton()->m_objectHighlightSelectionColorFg, 0.0f );
	}
	else if( isHighlighted )
	{
		_DrawObject( object, Options::GetSingleton()->m_objectHighlightColorBg, 2.0f );
		_DrawObject( object, Options::GetSingleton()->m_objectHighlightColorFg, 0.0f );
	}
	else if( isSelected )
	{
		_DrawObject( object, Options::GetSingleton()->m_objectSelectionColorBg, 2.0f );
		_DrawObject( object, Options::GetSingleton()->m_objectSelectionColorFg, 0.0f );
	}
	else
	{
		_DrawObject( object, Options::GetSingleton()->m_objectColor, 0.0f );
	}
}


//-------------------------------------------------------------------------------------------------
// _DrawLinedefs
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawLinedefs()
{
	/// Draw world linedefs
	std::list<MapLinedef*>::iterator begin = g_currentArea->m_linedefList.begin();
	std::list<MapLinedef*>::iterator end = g_currentArea->m_linedefList.end();
	std::list<MapLinedef*>::iterator iter;
	for( iter = begin; iter != end; ++ iter )
	{
		MapLinedef* linedef = (*iter);
		_DrawLinedef( *linedef );
	}

	/// Draw proto linedef if one exists
	if( m_newLineStart && m_newLineEnd )
	{
		Rgba protoColor = Options::GetSingleton()->m_protoLinedefColor;
		_DrawLinedef( m_newLineStart, m_newLineEnd, protoColor, 2.0f );
	}
}


//-------------------------------------------------------------------------------------------------
// _DrawVertexes
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawVertexes()
{
	/// Draw world vertexes
	std::list<MapVertex*>::iterator begin = g_currentArea->m_vertexList.begin();
	std::list<MapVertex*>::iterator end = g_currentArea->m_vertexList.end();
	std::list<MapVertex*>::iterator iter;
	for( iter = begin; iter != end; ++ iter )
	{
		MapVertex* vertex = (*iter);
		_DrawVertex( *vertex );
	}
}


//-------------------------------------------------------------------------------------------------
// _DrawObjects
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawObjects()
{
	/// Draw objects
	std::list<MapObject*>::iterator begin = g_currentArea->m_objectList.begin();
	std::list<MapObject*>::iterator end = g_currentArea->m_objectList.end();
	std::list<MapObject*>::iterator iter;
	for( iter = begin; iter != end; ++ iter )
	{
		MapObject* object = (*iter);
		_DrawObject( *object );
	}
}


//-------------------------------------------------------------------------------------------------
// _DrawWorld
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawWorld()
{
	_DrawVertexes();
	_DrawLinedefs();
	_DrawObjects();
}


//-------------------------------------------------------------------------------------------------
// GetEditModeDesc
//-------------------------------------------------------------------------------------------------
char* ViewXY::GetEditModeDesc()
{
	switch( m_editMode )
	{
		case EDIT_MODE_VERTEX_LINE_OBJECT:
			return "Edit Map Elements";
		case EDIT_MODE_LINE_CHAIN_CREATE:
			return "Line Chain Creation";
		case EDIT_MODE_EDIT_ELEMENT_PROPERTIES:
			return "Edit Element Properties";
		default:
			return "UNKNOWN!";
	}
}

//-------------------------------------------------------------------------------------------------
// _DrawText
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawText()
{
	char bottomOfWindowMessage[ 1024 ];
	sprintf( bottomOfWindowMessage, "Grid = %g%s, Mode = %s",
		m_grid->m_gridSize,
		m_grid->m_snapToGrid ? "" : "(off)",
		GetEditModeDesc() );

	theRenderer.PushCoordinateSystem();

	Vector2 topLeft( (float)m_mins.x, (float)m_mins.y );
	Vector2 bottomRight( (float)m_maxs.x, (float)m_maxs.y );
	Vector2 center = 0.5f * (topLeft + bottomRight);
	theRenderer.SetCoordinateSystem2d( topLeft, bottomRight );
	theRenderer.SetColor( Rgba::WHITE );
	const float fontAverageCharacterWidth = 5.0f; // FIXME: ask the renderer
	const float fontAverageCharacterHeight = 10.0f; // FIXME: ask the renderer
	float horizOffset = -0.5f * fontAverageCharacterWidth * (float)strlen( bottomOfWindowMessage );
	Vector2 mouseWorld = CalcWorldCoordsForCursor();
	Vector2 mouseGrid = CalcGridCoordsForCursor();
	theRenderer.XYPrintf( center.x + horizOffset + horizOffset, bottomRight.y - 20, "MouseWorld = (%g, %g), MouseGrid = (%g, %g)",
		mouseWorld.x, mouseWorld.y, mouseGrid.x, mouseGrid.y );
	theRenderer.XYPrintf( center.x + horizOffset, bottomRight.y - 4, "%s", bottomOfWindowMessage );
	theRenderer.XYPrintf( topLeft.x + fontAverageCharacterWidth, topLeft.y + fontAverageCharacterHeight, "2d View" );

	theRenderer.PopCoordinateSystem();
}


//-------------------------------------------------------------------------------------------------
// _DrawIcons
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawIcons()
{
	if( m_inputState == INPUT_STATE_ZOOMING )
	{
		_DrawIconZooming();
	}
	else if( m_inputState == INPUT_STATE_PANNING )
	{
		_DrawIconPanning();
	}
}


//-------------------------------------------------------------------------------------------------
// _DrawIconZooming
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawIconZooming()
{
	theRenderer.PushCoordinateSystem();

	Vector2 topLeft( (float)m_mins.x, (float)m_mins.y );
	Vector2 bottomRight( (float)m_maxs.x, (float)m_maxs.y );
	theRenderer.SetCoordinateSystem2d( topLeft, bottomRight );

	IntVector2 mousePos = theInputSystem.GetMousePosClientAbsolute();
	Vector2 center( (float)mousePos.x, (float)mousePos.y );
	float radius = 12.0f;

	Rgba iconColor = Options::GetSingleton()->m_iconColor;
	theRenderer.SetColor( iconColor );
	glLineWidth( 1.0f );
	glBegin( GL_LINES );
	{
		glVertex2f( center.x - radius, center.y - radius );
		glVertex2f( center.x + radius, center.y - radius );

		glVertex2f( center.x + radius, center.y - radius );
		glVertex2f( center.x - radius, center.y + radius );

		glVertex2f( center.x - radius, center.y + radius );
		glVertex2f( center.x + radius, center.y + radius );

		glVertex2f( center.x - (0.5f * radius), center.y );
		glVertex2f( center.x + (0.5f * radius), center.y );
	}
	glEnd(); // GL_LINES
	
	theRenderer.PopCoordinateSystem();
}


//-------------------------------------------------------------------------------------------------
// _DrawIconPanning
//-------------------------------------------------------------------------------------------------
void ViewXY::_DrawIconPanning()
{
	theRenderer.PushCoordinateSystem();

	Vector2 topLeft( (float)m_mins.x, (float)m_mins.y );
	Vector2 bottomRight( (float)m_maxs.x, (float)m_maxs.y );
	theRenderer.SetCoordinateSystem2d( topLeft, bottomRight );

	IntVector2 mousePos = theInputSystem.GetMousePosClientAbsolute();
	Vector2 center( (float)mousePos.x, (float)mousePos.y );
	float radius = 16.0f;
	float lip = radius * 0.25f;

	Rgba iconColor = Options::GetSingleton()->m_iconColor;
	theRenderer.SetColor( iconColor );
	glLineWidth( 1.0f );
	glBegin( GL_LINES );
	{
		glVertex2f( center.x - radius, center.y );
		glVertex2f( center.x + radius, center.y );

		glVertex2f( center.x, center.y - radius );
		glVertex2f( center.x, center.y + radius );

		glVertex2f( center.x - radius, center.y );
		glVertex2f( center.x - radius + lip, center.y - lip );

		glVertex2f( center.x - radius, center.y );
		glVertex2f( center.x - radius + lip, center.y + lip );

		glVertex2f( center.x + radius, center.y );
		glVertex2f( center.x + radius - lip, center.y - lip );

		glVertex2f( center.x + radius, center.y );
		glVertex2f( center.x + radius - lip, center.y + lip );

		glVertex2f( center.x, center.y - radius );
		glVertex2f( center.x - lip, center.y - radius + lip );

		glVertex2f( center.x, center.y - radius );
		glVertex2f( center.x + lip, center.y - radius + lip );

		glVertex2f( center.x, center.y + radius );
		glVertex2f( center.x - lip, center.y + radius - lip );

		glVertex2f( center.x, center.y + radius );
		glVertex2f( center.x + lip, center.y + radius - lip );
	}
	glEnd(); // GL_LINES
	
	theRenderer.PopCoordinateSystem();
}


//-------------------------------------------------------------------------------------------------
// ProcessInputEvent
//-------------------------------------------------------------------------------------------------
VIRTUAL bool ViewXY::ProcessInputEvent( TheInputSystem::inputEventType_e eventType, byte_t key )
{
	bool wasHandled = false;

	if( key == VK_LBUTTON )
	{
		if( m_isLMBDrag && eventType == TheInputSystem::INPUT_EVENT_KEYUP )
		{
			_OnFinishLMBDrag();
		}
		m_isLMBDrag = false;
	}
	else if( key == VK_RBUTTON )
	{
		if( m_isRMBDrag && eventType == TheInputSystem::INPUT_EVENT_KEYUP )
		{
			_OnFinishRMBDrag();
		}
		m_isRMBDrag = false;
	}

	wasHandled = ProcessInputEventContextSpecific( eventType, key );
	if( !wasHandled )
	{
		wasHandled = ProcessInputEventContextGeneral( eventType, key );
	}

	return wasHandled;
}


//-------------------------------------------------------------------------------------------------
// ProcessInputEventContextSpecific_VertexLineObject
//-------------------------------------------------------------------------------------------------
bool ViewXY::ProcessInputEventContextSpecific_VertexLineObject( TheInputSystem::inputEventType_e eventType, byte_t key )
{
	bool wasHandled = false;

	if( key == VK_RBUTTON )
	{
		if( eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
		{
			theInputSystem.UseWindowsMouseCursor( false );
			wasHandled = true;
		}
		else if( eventType == TheInputSystem::INPUT_EVENT_KEYUP )
		{
			theInputSystem.UseWindowsMouseCursor( true );
			wasHandled = true;
		}
	}

	if( key == VK_LBUTTON && eventType == TheInputSystem::INPUT_EVENT_KEYREPEAT )
	{
		// LMB Double-click
		StartNewLineChain();
		wasHandled = true;
	}
	else if( key == VK_RBUTTON && eventType == TheInputSystem::INPUT_EVENT_KEYREPEAT )
	{
		// RMB Double-click
		CreateNewObject();
		wasHandled = true;
	}
	else if( key == VK_DELETE && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		// Hit DEL
		g_currentArea->DeleteHighlightedAndSelected();
		wasHandled = true;
	}
	else if( key == VK_LBUTTON && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		VertexLineLMBDown();
		wasHandled = true;
	}
	else if( key == VK_LBUTTON && eventType == TheInputSystem::INPUT_EVENT_KEYUP )
	{
		VertexLineLMBUp();
		wasHandled = true;
	}
	else if( key == VK_ESCAPE && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		// Hit ESC
		g_currentArea->ClearSelection();
		wasHandled = true;
	}
	else if( key == 'U' && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		theEditor->LoadUnderlayTexture( "data/UnderlayTest.tga" );
		wasHandled = true;
	}
	else if( key == VK_OEM_COMMA && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		theEditor->m_underlayBrightness -= 0.1f;
		theEditor->m_underlayBrightness = ClampFloat( theEditor->m_underlayBrightness, 0.0f, 1.0f );
		wasHandled = true;
	}
	else if( key == VK_OEM_PERIOD && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		theEditor->m_underlayBrightness += 0.1f;
		theEditor->m_underlayBrightness = ClampFloat( theEditor->m_underlayBrightness, 0.0f, 1.0f );
		wasHandled = true;
	}

	return wasHandled;
}


//-------------------------------------------------------------------------------------------------
// ProcessInputEventContextSpecific_LineChainCreate
//-------------------------------------------------------------------------------------------------
bool ViewXY::ProcessInputEventContextSpecific_LineChainCreate( TheInputSystem::inputEventType_e eventType, byte_t key )
{
	bool wasHandled = false;

	if( key == VK_RBUTTON )
	{
		if( eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
		{
			theInputSystem.UseWindowsMouseCursor( false );
			wasHandled = true;
		}
		else if( eventType == TheInputSystem::INPUT_EVENT_KEYUP )
		{
			theInputSystem.UseWindowsMouseCursor( true );
			wasHandled = true;
		}
	}

	if( key == VK_LBUTTON && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		// Single left click
		ExtendLineChain();
		wasHandled = true;
	}
	else if( key == VK_ESCAPE && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		// Hit ESC
		TerminateLineChain();
		wasHandled = true;
	}
	else if( key == VK_RBUTTON && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		// RMB down
		LineChainRMBDown();
		wasHandled = true;
	}
	else if( key == VK_RBUTTON && eventType == TheInputSystem::INPUT_EVENT_KEYUP )
	{
		// RMB up
		LineChainRMBUp();
		wasHandled = true;
	}

	return wasHandled;
}


//-------------------------------------------------------------------------------------------------
// ProcessInputEventContextSpecific_EditElementProperties
//-------------------------------------------------------------------------------------------------
bool ViewXY::ProcessInputEventContextSpecific_EditElementProperties( TheInputSystem::inputEventType_e eventType, byte_t key )
{
	if( key == VK_ESCAPE && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		// Hit ESC
		TerminateEditProperties();
		return true;
	}

	return false;
}


//-------------------------------------------------------------------------------------------------
// ProcessInputEventContextSpecific
//-------------------------------------------------------------------------------------------------
bool ViewXY::ProcessInputEventContextSpecific( TheInputSystem::inputEventType_e eventType, byte_t key )
{
	switch( m_editMode )
	{
		case EDIT_MODE_VERTEX_LINE_OBJECT:
			return ProcessInputEventContextSpecific_VertexLineObject( eventType, key );
			break;
		case EDIT_MODE_LINE_CHAIN_CREATE:
			return ProcessInputEventContextSpecific_LineChainCreate( eventType, key );
			break;
		case EDIT_MODE_EDIT_ELEMENT_PROPERTIES:
			return ProcessInputEventContextSpecific_EditElementProperties( eventType, key );
			break;
	}

	return false;
}



//-------------------------------------------------------------------------------------------------
// ProcessInputEventContextGeneral
//-------------------------------------------------------------------------------------------------
bool ViewXY::ProcessInputEventContextGeneral( TheInputSystem::inputEventType_e eventType, byte_t key )
{
	if( key == 'G' && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		switch( m_gridDrawMode )
		{
			case GRID_DRAW_NONE:	m_gridDrawMode = GRID_DRAW_LINES;	break;
			case GRID_DRAW_LINES:	m_gridDrawMode = GRID_DRAW_DOTS;	break;
			case GRID_DRAW_DOTS:	m_gridDrawMode = GRID_DRAW_NONE;	break;
		}
		return true;
	}

	if( key == 'S' && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		m_grid->m_snapToGrid = !m_grid->m_snapToGrid;
		return true;
	}

	if( key == VK_LEFTBRACKET && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		m_grid->DecreaseGridSize();
		return true;
	}

	if( key == VK_RIGHTBRACKET && eventType == TheInputSystem::INPUT_EVENT_KEYDOWN )
	{
		m_grid->IncreaseGridSize();
		return true;
	}

	return false;
}


//-------------------------------------------------------------------------------------------------
// CreateNewObject
//-------------------------------------------------------------------------------------------------
void ViewXY::CreateNewObject()
{
	Vector2 gridCoords = CalcGridCoordsForCursor();
//	m_editMode = EDIT_MODE_EDIT_ELEMENT_PROPERTIES;
	m_editMode = EDIT_MODE_VERTEX_LINE_OBJECT;
	MapObject* newObject = g_currentArea->CreateNewObject( gridCoords );
	m_editPropertiesElementList.clear();
	m_editPropertiesElementList.push_back( newObject );
}


//-------------------------------------------------------------------------------------------------
// StartNewLineChain
//-------------------------------------------------------------------------------------------------
void ViewXY::StartNewLineChain()
{
	/// Create a new vertex and enter "linedef chain create" mode
	m_editMode = EDIT_MODE_LINE_CHAIN_CREATE;
	Vector2 gridCoords = CalcGridCoordsForCursor();

	m_newLineStart = g_currentArea->CreateNewVertex( gridCoords, false );
	m_newLineEnd = g_currentArea->CreateNewVertex( gridCoords, true );
}


//-------------------------------------------------------------------------------------------------
// ExtendLineChain
//-------------------------------------------------------------------------------------------------
void ViewXY::ExtendLineChain()
{
	/// Check if proto linedef has a reasonably large length
	if( IsDistanceBetweenPointsLess( m_newLineStart->m_pos, m_newLineEnd->m_pos, Grid::GetGridEpsilon() ) )
	{
		return;
	}

	/// Integrate the end vertex into the world
	m_newLineEnd = g_currentArea->IntegrateVertexIntoWorld( m_newLineEnd );

	/// Make the current proto linedef part of the world and start a new proto linedef
	g_currentArea->CreateNewLinedef( m_newLineStart, m_newLineEnd );
	m_newLineStart = m_newLineEnd;
	m_newLineEnd = g_currentArea->CreateNewVertex( CalcGridCoordsForCursor(), true );
}


//-------------------------------------------------------------------------------------------------
// LineChainRMBDown
//-------------------------------------------------------------------------------------------------
void ViewXY::LineChainRMBDown()
{
	m_isPotentialRMBClick = true;
}


//-------------------------------------------------------------------------------------------------
// LineChainRMBUp
//-------------------------------------------------------------------------------------------------
void ViewXY::LineChainRMBUp()
{
	if( m_isPotentialRMBClick )
	{
		m_isPotentialRMBClick = false;

		/// Make the proto line permanent and then terminate the chain
		ExtendLineChain();
		TerminateLineChain();
	}
}


//-------------------------------------------------------------------------------------------------
// VertexLineLMBDown
//-------------------------------------------------------------------------------------------------
void ViewXY::VertexLineLMBDown()
{
	m_isPotentialLMBClick = true;
}


//-------------------------------------------------------------------------------------------------
// VertexLineLMBUp
//-------------------------------------------------------------------------------------------------
void ViewXY::VertexLineLMBUp()
{
	if( m_isPotentialLMBClick )
	{
		m_isPotentialLMBClick = false;

		/// Toggle selection of the currently highlighted object(s)
		g_currentArea->ToggleSelectionOfHighlightedElements();
	}
}


//-------------------------------------------------------------------------------------------------
// TerminateLineChain
//-------------------------------------------------------------------------------------------------
void ViewXY::TerminateLineChain()
{
	/// Cancel the current proto linedef and return to normal editing mode
	g_currentArea->DeleteMapVertex( m_newLineEnd );
	if( m_newLineStart->m_linedefs.size() == 0 )
	{
		g_currentArea->DeleteMapVertex( m_newLineStart );
	}
	m_editMode = EDIT_MODE_VERTEX_LINE_OBJECT;

	m_newLineStart = NULL;
	m_newLineEnd = NULL;
}


//-------------------------------------------------------------------------------------------------
// TerminateEditProperties
//-------------------------------------------------------------------------------------------------
void ViewXY::TerminateEditProperties()
{
	m_editMode = EDIT_MODE_VERTEX_LINE_OBJECT;
	m_editPropertiesElementList.clear();
}


//-------------------------------------------------------------------------------------------------
// CalcWorldCoordsForScreenCoords
//-------------------------------------------------------------------------------------------------
const Vector2 ViewXY::CalcWorldCoordsForScreenCoords( const IntVector2& screenCoords )
{
	/// Convert screen coords to view-parameterized in the range [0,1]
	Vector2 normalizedScreenCoords;
	normalizedScreenCoords.x = (float)(screenCoords.x - m_mins.x) / (float)(m_maxs.x - m_mins.x);
	normalizedScreenCoords.y = 1.0f - (float)(screenCoords.y - m_mins.y) / (float)(m_maxs.y - m_mins.y);

	Vector2 viewWorldSize = m_viewWorldMaxs - m_viewWorldMins;
	Vector2 worldCoords = viewWorldSize;
	worldCoords.x *= normalizedScreenCoords.x;
	worldCoords.y *= normalizedScreenCoords.y;
	worldCoords += m_viewWorldMins;
	return worldCoords;
}


//-------------------------------------------------------------------------------------------------
// CalcGridCoordsForScreenCoords
//-------------------------------------------------------------------------------------------------
const Vector2 ViewXY::CalcGridCoordsForScreenCoords( const IntVector2& screenCoords )
{
	Vector2 worldCoords = CalcWorldCoordsForScreenCoords( screenCoords );
	Vector2 gridCoords = m_grid->FindClosestGridPointToWorldCoordinates( worldCoords );
	return gridCoords;
}


//-------------------------------------------------------------------------------------------------
// CalcWorldCoordsForCursor
//-------------------------------------------------------------------------------------------------
const Vector2 ViewXY::CalcWorldCoordsForCursor()
{
	IntVector2 mouseCoords = theInputSystem.GetMousePosClientAbsolute();
	return CalcWorldCoordsForScreenCoords( mouseCoords );
}


//-------------------------------------------------------------------------------------------------
// CalcGridCoordsForCursor
//-------------------------------------------------------------------------------------------------
const Vector2 ViewXY::CalcGridCoordsForCursor()
{
	IntVector2 mouseCoords = theInputSystem.GetMousePosClientAbsolute();
	return CalcGridCoordsForScreenCoords( mouseCoords );
}


//-------------------------------------------------------------------------------------------------
// AlignWorldCoordsToScreenCoords
//-------------------------------------------------------------------------------------------------
void ViewXY::AlignWorldCoordsToScreenCoords( const Vector2& worldCoords, const IntVector2& screenCoords )
{
	Vector2 oldWorldCoords = CalcWorldCoordsForScreenCoords( screenCoords );
	Vector2 translation = worldCoords - oldWorldCoords;
	m_viewCenterWorldCoords += translation;
	UpdateViewWorldMinsMaxs();
}



