//-------------------------------------------------------------------------------------------------
// ViewXY.hpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_ViewXY_
#define _include_ViewXY_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "Grid.hpp"
#include "ViewBase.hpp"
#include "MapArea.hpp"
#include "gameCommon.hpp"


const float DEFAULT_SCALE_XY = 12.0f;
const float DEFAULT_UNSCALED_WORLD_WIDTH = 128.0f;


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		ViewXY
// 
///////////////////////////////////////////////////////////////////////////////////////////////////
class ViewXY : public ViewBase
{
private:
public:
	enum GridDrawMode
	{
		GRID_DRAW_NONE,
		GRID_DRAW_LINES,
		GRID_DRAW_DOTS,
		NUM_GRID_MODES,
	};

	enum InputState
	{
		INPUT_STATE_NORMAL,
		INPUT_STATE_PANNING,
		INPUT_STATE_ZOOMING,
	};

	enum EditMode
	{
		EDIT_MODE_VERTEX_LINE_OBJECT,
		EDIT_MODE_LINE_CHAIN_CREATE,
		EDIT_MODE_EDIT_ELEMENT_PROPERTIES,
	};

	Vector2					m_viewCenterWorldCoords;
	Vector2					m_viewWorldMins;
	Vector2					m_viewWorldMaxs;
	Vector2					m_cursorGridCoordsOfLMBDragStart;
	Vector2					m_cursorGridCoordsOfRMBDragStart;
	Vector2					m_cursorGridCoordsOfLastSelectionDrag;
	float					m_scale;
	Grid*					m_grid;
	GridDrawMode			m_gridDrawMode;
	InputState				m_inputState;
	static EditMode			m_editMode;

	// hmm... should this stuff go here?
	static MapVertex*		m_newLineStart;
	static MapVertex*		m_newLineEnd;
	bool					m_isPotentialLMBClick;
	bool					m_isPotentialRMBClick;
	bool					m_isLMBDrag;
	bool					m_isRMBDrag;
	MapElement*				m_highlightedItemAtStartOfDrag;
	bool					m_highlightedItemWasSelectedAtStartOfDrag;
	std::list<MapElement*>	m_editPropertiesElementList;

	ViewXY();
	~ViewXY();

	virtual void Draw();
	virtual void Update();
	virtual bool ProcessInputEvent( TheInputSystem::inputEventType_e eventType, byte_t key );
	bool ProcessInputEventContextGeneral( TheInputSystem::inputEventType_e eventType, byte_t key );
	bool ProcessInputEventContextSpecific( TheInputSystem::inputEventType_e eventType, byte_t key );
	bool ProcessInputEventContextSpecific_VertexLineObject( TheInputSystem::inputEventType_e eventType, byte_t key );
	bool ProcessInputEventContextSpecific_LineChainCreate( TheInputSystem::inputEventType_e eventType, byte_t key );
	bool ProcessInputEventContextSpecific_EditElementProperties( TheInputSystem::inputEventType_e eventType, byte_t key );
	char* GetEditModeDesc();
	void CreateNewObject();
	void StartNewLineChain();
	void ExtendLineChain();
	void LineChainRMBDown();
	void LineChainRMBUp();
	void VertexLineLMBDown();
	void VertexLineLMBUp();
	void TerminateLineChain();
	void TerminateEditProperties();
	void _DrawUnderlay();
	void _DrawGrid();
	void _DrawGridLines( const Vector2& gridMins, const Vector2& gridMaxs );
	void _DrawGridDots( const Vector2& gridMins, const Vector2& gridMaxs );
	void _DrawWorld();
	void _DrawLinedefs();
	void _DrawVertexes();
	void _DrawObjects();
	void _DrawLinedef( const MapVertex* start, const MapVertex* end, const Rgba& color, float thickness );
	void _DrawLinedef( const MapLinedef& linedef );
	void _DrawVertex( const MapVertex& vertex, const Rgba& color, float size );
	void _DrawVertex( const MapVertex& vertex );
	void _DrawObject( const MapObject& object, const Rgba& color, float paddingSize );
	void _DrawObject( const MapObject& object );
	void _DrawText();
	void _DrawIcons();
	void _DrawIconZooming();
	void _DrawIconPanning();
	void _SetCoordinateSystem();
	void _UpdateZoom();
	void _UpdatePan();
	void _UpdateHighlights();
	void _UpdateModeAll();
	void _UpdateSelectionDrag();
	void _UpdateModeVertexLineObject();
	void _UpdateModeLineChainCreate();
	void _UpdateModeEditElementProperties();
	void _OnStartLMBDrag();
	void _OnStartRMBDrag();
	void _OnFinishLMBDrag();
	void _OnFinishRMBDrag();
	void UpdateViewWorldMinsMaxs();
	float CalcPixelsPerGrid();
	const Vector2 CalcWorldCoordsForScreenCoords( const IntVector2& screenCoords );
	const Vector2 CalcGridCoordsForScreenCoords( const IntVector2& screenCoords );
	const Vector2 CalcWorldCoordsForCursor();
	const Vector2 CalcGridCoordsForCursor();
	void AlignWorldCoordsToScreenCoords( const Vector2& worldCoords, const IntVector2& screenCoords );
};


#endif // _include_ViewXY_
